/*
 *
 * observer_i.h
 *
 */

#ifndef OBSERVER_I_H
#define OBSERVER_I_H





inline Observer::Observer(const WPoint pos, const Number angle, 
			  const Number dist_obs_scr, const int width)
{
  _pos = pos;
  _angle = angle;
  _width = width;
  _dist_obs_scr = dist_obs_scr;
  compute_orig_launch_table();
  compute_launch_table();
}

inline void Observer::SetWidth(int width)
{
  _dist_obs_scr = (_dist_obs_scr * width) / _width;
  _width = width;
  compute_orig_launch_table();
  compute_launch_table();
}

inline int Observer::GetWidth(void)
{
  return _width;
}
  
inline void Observer::Move(WVector V)
{
  _pos += &V;
  _last_move = V;
}

inline void Observer::Move(Number amp)
{
  _last_move = _launch_table[_width >> 1];
  _last_move.x *= amp;
  _last_move.y *= amp;
  _pos += &_last_move;
}

inline void Observer::GetMove(Number amp, WRay *R)
{
  R->P  = _pos;
  R->V  = _launch_table[_width >> 1];
  R->V.x *= amp;
  R->V.y *= amp;
}

inline void Observer::Rotate(Number angle)
{
  _angle += angle;
  compute_launch_table();
}

inline void Observer::SetAngle(Number angle)
{
  _angle = angle;
  compute_launch_table();
}

inline void Observer::Launch(int x, WRay *R)
{
  R->P = _pos;
  R->V = _launch_table[x];
}

inline void Observer::compute_orig_launch_table()
{
  for(int i = 0; i < _width; i++)
    {
      _orig_launch_table[i].x = i - (_width >> 1);
      _orig_launch_table[i].y = _dist_obs_scr;
      _orig_launch_table[i].Normalize();
    }
}

inline void Observer::compute_launch_table()
{
  for(int i = 0; i < _width; i++)
    {
      Number s = WSin(_angle);
      Number c = WCos(_angle);

      _launch_table[i].x = c;
      _launch_table[i].x *= _orig_launch_table[i].x;
      _launch_table[i].x.mac( s,_orig_launch_table[i].y);

      _launch_table[i].y = s;
      _launch_table[i].y *= _orig_launch_table[i].x;
      _launch_table[i].y.neg();
      _launch_table[i].y.mac( c,_orig_launch_table[i].y);

    }
}


inline int  Observer::WallHeight(int x, Number t)
{

  Number y = t;
  y *= _orig_launch_table[x].y;

  Number h = 73.666;
  h *= _dist_obs_scr;
  h /= y;

  return (int)h;
}

#endif
