
/*
 *
 * wdefs_i.h
 *
 */

#ifndef WDEFS_I_H
#define WDEFS_I_H

#include <math.h>

inline WPoint::WPoint(void) {}

inline WPoint::WPoint(const Number xx, const Number yy)
{
  x = xx;
  y = yy;
}

inline WPoint& WPoint::operator += (const WVector *V)
{
  x += V->x;
  y += V->y;
  return *this;
}


inline WVertex::WVertex(void) {};

inline WVertex::WVertex(const Number xx, const Number yy) : WPoint(xx,yy)
{
}

inline WVertex::WVertex(const Number xx, const Number yy, 
			const int c1, const int c2) : WPoint(xx,yy)
{
  _lower_col = c1;
  _upper_col = c2;
}

inline WVector::WVector(void) {}

inline WVector::WVector(const Number xx, const Number yy)
{
  x = xx;
  y = yy;
}

inline WVector::WVector(const WPoint &P1, const WPoint &P2)
{
  x = P2.x;
  x -= P1.x;
  y = P2.y ;
  y -= P1.y;
}


inline Number operator*(WVector &V1, WVector &V2)
{
  Number x = V1.x;
  x *= V2.x;
  x.mac(V1.y, V2.y);
  return x;
}

inline Number operator%(WVector &V1, WVector &V2) // Vincent's product
{
  Number x;

  x.mld(V1.x, V2.y);
  x.msc(V1.y, V2.x);

  return x;
}

inline void WVector::Normalize(void)
{
  double xx = x;
  double yy = y;
  double n = sqrt(xx*xx + yy*yy);
  if(n)
    {
      x = xx/n;
      y = yy/n;
    }
}

inline WRay::WRay(void) {};
inline WRay::WRay(const WPoint& PP, const WVector& VV)
{
  P = PP;
  V = VV;
}

inline IsectInfo::IsectInfo(void)
{
  flag = 0;
}

inline Wall::Wall(int idx1, int idx2, int color)
{
  _idx1  = idx1;
  _idx2  = idx2;
  _color = color;
  _flags = 0;
}


inline Wall::Wall(void) 
{
}


inline void Wall::Setup(array<WVertex> &vertice) 
{
  _P1    = &vertice[_idx1];
  _P2    = &vertice[_idx2];
  _P1P2  = WVector(*_P1, *_P2);
}

inline void Wall::SetFlags(int flags)
{
  _flags = flags;
}

inline int Wall::Isect(WRay &R)
{
  Number D,U;

  D = _P1P2 % R.V;

  if(D.le())
      return 0;

  WVector PP1(R.P, *_P1);

  U = R.V % PP1;

  if(U.lw() || (U >= D))
      return 0;

  Number t = _P1P2 % PP1;

  if(t.lw() || t >= D)
      return 0;

  return 1;
}


inline int Wall::Isect(WRay &R, IsectInfo &II)
{

  if(_flags & W_WINDOW) // Windows let pass light rays through
    return 0;           // (but not observers ...)

  Number D,U;
  D = _P1P2 % R.V;

  if(D.le())
      return 0;

  WVector PP1(R.P, *_P1);

  U = R.V % PP1;


  if(U.lw() || (U > D))
      return 0;

  Number t = _P1P2 % PP1;

  t /= D;

  if(t < 0)
      return 0;

  if(II.flag && (t > II.t))
      return 0;
  
  II.t = t;
  II.flag = 1;
  
  II.u = U;
  II.u /= D;

  II.wall = this;

  return 1;
}

#endif

