/*
 *
 * world.C
 *
 */

#include "world.h"

void World::Draw(Observer& O, GraphicPort *GP)
{
  ColorIndex Ceilings  = GP->Colormap()[BLUE];
  ColorIndex Floors    = GP->Colormap()[GREEN];
  ColorIndex Walls;

  int x, w;
  WRay      R;
  char     *middle_ptr = GP->GraphMem() + GP->Width() * (GP->Height() >> 1);
  char     *higher_ptr = NULL, *lower_ptr = NULL;


  for(x=0; x < O.GetWidth(); x++)
    {
      IsectInfo II;
      II.flag = 0;

      O.Launch(x, &R);
      for(w = 0; w < _wall.Size(); w++)
	  _wall[w].Isect(R,II);
      
      if(II.flag)
	{
	  Number color   = II.wall->_P1->_lower_col;
	  color.mac(II.wall->_coldiff, II.u);
	  Walls = GP->Colormap()[(int)color];
	}
      else
	Walls = 0;


      int H = II.flag ? (O.WallHeight(x,II.t) >> 1) : 0;
      higher_ptr = middle_ptr;
      lower_ptr  = middle_ptr - GP->BytesPerLine();
      for(int i=0; i < (GP->Height() >> 1); i++)
	{
	  if(i < H)
	    *higher_ptr = *lower_ptr = Walls;
	  else
	    {
	      *higher_ptr = Floors;
	      *lower_ptr  = Ceilings;
	    }
	  lower_ptr  -= GP->BytesPerLine();
	  higher_ptr += GP->BytesPerLine();
	}
      middle_ptr++;
    }
}

int World::Isect(WRay &R)
{
  int i;
  for(i=0; i < _wall.Size(); i++)
    if(_wall[i].Isect(R))
      return 1;

  return 0;
}

void World::Lighting(WVector& Light)
{
  WVector& L = Light;
  int i,j;
  int nbr;

  L.Normalize();

  for(i=0; i<_wall.Size(); i++)
    {
      Number x = _wall[i]._P1P2.y;
      Number y = _wall[i]._P1P2.x;
      y.neg();
      WVector N(x,y);
      N.Normalize();

      Number I = L * N;
      I *= 63;

//    cerr << i << "> " << I << "\n";

      _wall[i]._color = I.gt() ? 0 : -(int)I;
    }


//  cerr << "----------------------\n";

  for(i=0; i<_vertex.Size(); i++)
    {
      _vertex[i]._lower_col = (nbr = 0);
      for(j=0; j<_wall.Size(); j++)
	if((_wall[j]._P1 == &_vertex[i]) || (_wall[j]._P2 == &_vertex[i]))
	  {
	    _vertex[i]._lower_col += _wall[j]._color;
	    nbr++;
	  }
      if(nbr)
	_vertex[i]._lower_col /= nbr;

//      cerr << i << "> " <<  _vertex[i]._lower_col << "\n";

    }

  for(i=0; i<_wall.Size(); i++)
      _wall[i]._coldiff = _wall[i]._P2->_lower_col - _wall[i]._P1->_lower_col;

}
