/*
 *
 * gobj.h
 *
 */

#ifndef GOBJ_H
#define GOBJ_H

#include "sintab.h"
#include "gdefs.h"
#include "polyeng.h"

//
// For the moment, GraphicObjects need to implement
// some of the 3Dengine stuff. It is implemented as
// static members. It will be moved to the right
// class later on ...
//

const int M_BIG   = 16384;
const int M_SHIFT = 14;

class MVector
{
 public:
  MVector(int xx = 0, int yy = 0, int zz = 0);
  void Normalize(void);
  void RotX(Angle r);
  void RotY(Angle r);
  void RotZ(Angle r);
  void Rotate(Angle rx, Angle ry, Angle rz);
  void Translate(int tx, int ty, int tz);
  void Scale(double sx, double sy, double sz);
  void Scale(double s);
  
  int x,y,z;
};

// istream& operator >> (istream& input,  MVector& V);
// ostream& operator << (ostream& output, MVector& V);




class MVertex : public MVector
{
 public:
  MVertex(int  x = 0, int y = 0, int z = 0);
  MVector N;
  Vertex  Projection;
  int r,g,b;
  int count;
};




class GraphicObject
{

 public:
  virtual ~GraphicObject();
  virtual void Draw(PolygonEngine *PE) = 0;
  virtual void RotX(Angle r) = 0;
  virtual void RotY(Angle r) = 0;
  virtual void RotZ(Angle r) = 0;
  void Rotate(char axis, Angle r);
  void Rotate(Angle rx, Angle ry, Angle rz);
  virtual void Translate(int tx, int ty, int tz)      = 0;
  virtual void Scale(double sx, double sy, double sz) = 0;
  void Scale(double s);
  virtual void Lighting(void) = 0;



  static void SetGeometry(int width, int height);

  static void SetLightDirection(int  Lx, int  Ly, int  Lz);
  static void GetLightDirection(int *Lx, int *Ly, int *Lz);

  static void SetLightColor(int  Lr, int  Lg, int  Lb);
  static void GetLightColor(int *Lr, int *Lg, int *Lb);

  static void SetLightColor(int  Lc);
  static void GetLightColor(int *Lc);

  static void SetZoom(int  zoom);
  static int  GetZoom(void);
  static void SetZoomStep(int zoom_step);
  static int  GetZoomStep(void);
  static void ZoomIn(void);
  static void ZoomOut(void);
  static MVector& LightDirection(void);

 protected:

  static void Project(MVertex *V);

  static int _width;
  static int _height;
  static MVector _L;
  static int _Lr, _Lg, _Lb;
  static int _Lc;

  static int _zoom;
  static int _zoom_step;

};

// PolygonEngine* operator << (PolygonEngine* output, GraphicObject& O);

#include "gobj_i.h"

#endif
