/*
 *
 * mesh.C
 *
 */


#include "mesh.h"
#include <iostream.h>
#include <string.h>

// I do not use MVector::RotX cause it would call Sin() and Cos() 
// for each vertex ...

typedef struct
{
  double x,y,z;
} VectorF;

typedef struct
{
  double r,g,b;
} ColorF;

void Mesh::RotX(Angle r)
{
  int i;
  long s = Sin(r),c = Cos(r);
  long t1, t2;


  for(i=0; i < _nvertex; i++)
    {
      t1 = _vertex[i].y;
      t2 = _vertex[i].z;
      _vertex[i].y = ( c * t1 + s * t2) >> SIN_SHIFT;
      _vertex[i].z = (-s * t1 + c * t2) >> SIN_SHIFT;

      if(_resources.Get(MR_SMOOTH))
	{
	  t1 = _vertex[i].N.y;
	  t2 = _vertex[i].N.z;
	  _vertex[i].N.y = ( c * t1 + s * t2) >> SIN_SHIFT;
	  _vertex[i].N.z = (-s * t1 + c * t2) >> SIN_SHIFT;
	}
    }

  for(i=0; i < _nface; i++)
    {
      t1 = _face[i].N.y;
      t2 = _face[i].N.z;
      _face[i].N.y = ( c * t1 + s * t2) >> SIN_SHIFT;
      _face[i].N.z = (-s * t1 + c * t2) >> SIN_SHIFT;
    }
}


void Mesh::RotY(Angle r)
{
  int i;
  long s = Sin(r),c = Cos(r);
  long t1,t2;

  for(i=0; i < _nvertex; i++)
    {
      t1 = _vertex[i].x;
      t2 = _vertex[i].z;
      _vertex[i].x = ( c * t1 + s * t2) >> SIN_SHIFT;
      _vertex[i].z = (-s * t1 + c * t2) >> SIN_SHIFT;

      if(_resources.Get(MR_SMOOTH))
	{
	  t1 = _vertex[i].N.x;
	  t2 = _vertex[i].N.z;
	  _vertex[i].N.x = ( c * t1 + s * t2) >> SIN_SHIFT;
	  _vertex[i].N.z = (-s * t1 + c * t2) >> SIN_SHIFT;
	}
    }

  for(i=0; i < _nface; i++)
    {
      t1 = _face[i].N.x;
      t2 = _face[i].N.z;
      _face[i].N.x = ( c * t1 + s * t2) >> SIN_SHIFT;
      _face[i].N.z = (-s * t1 + c * t2) >> SIN_SHIFT;
    }
}

void Mesh::RotZ(Angle r)
{
  int i;
  long s = Sin(r),c = Cos(r);
  long t1, t2;

  for(i=0; i < _nvertex; i++)
    {
      t1 = _vertex[i].x;
      t2 = _vertex[i].y;
      _vertex[i].x = ( c * t1 + s * t2) >> SIN_SHIFT;
      _vertex[i].y = (-s * t1 + c * t2) >> SIN_SHIFT;

      if(_resources.Get(MR_SMOOTH))
	{
	  t1 = _vertex[i].N.x;
	  t2 = _vertex[i].N.y;
	  _vertex[i].N.x = ( c * t1 + s * t2) >> SIN_SHIFT;
	  _vertex[i].N.y = (-s * t1 + c * t2) >> SIN_SHIFT;
	}
    }

  for(i=0; i < _nface; i++)
    {
      t1 = _face[i].N.x;
      t2 = _face[i].N.y;
      _face[i].N.x = ( c * t1 + s * t2) >> SIN_SHIFT;
      _face[i].N.y = (-s * t1 + c * t2) >> SIN_SHIFT;
    }
}

void Mesh::Translate(int tx, int ty, int tz)
{
  int i;
  for(i=0; i<_nvertex; i++)
    _vertex[i].Translate(tx, ty, tz);
}

void Mesh::Scale(double sx, double sy, double sz)
{
  int i;
  for(i=0; i<_nvertex; i++)
    _vertex[i].Scale(sx, sy, sz);
}


void Mesh::Setup(PolygonEngine *PE)
{
  SetGeometry(PE->Width(), PE->Height());
  

  switch(PE->BitsPerPixel())
    {
    case 8:
      memset(PE->GraphMem(), PE->Colormap()[0],
	     PE->BytesPerLine() * PE->Height());
      break;
    case 24:
      memset(PE->GraphMem(), 4,
	     PE->BytesPerLine() * PE->Height());
      break;
    case 15:
    case 16:
      unsigned short *p = (unsigned short *)PE->GraphMem();
      const unsigned short c = 1 | (1 << 5) | (1 << 10);
      for(int i = (PE->BytesPerLine() * PE->Height()) >> 1; i; i--)
	*(p++) = c;
      break;
    }

  if(!_flags.Get(MF_CONVEX))
    memset(PE->ZMem(), 255, PE->Width() * PE->Height() * sizeof(ZCoord));
}

void Mesh::Draw(PolygonEngine *PE)
{
  int i,j;

  PE->PushAttributes();
  
  if(_flags.Get(MF_SMOOTH) || _flags.Get(MF_BLEND))
    PE->Attributes().Set(GA_GOURAUD);
  else
    PE->Attributes().Reset(GA_GOURAUD);

  if((!_flags.Get(MF_CONVEX)))
    PE->Attributes().Set(GA_ZBUFFER);
  else
    PE->Attributes().Reset(GA_ZBUFFER);


  PE->CommitAttributes();


  for(i=0; i<_nvertex; i++)
    Project(&_vertex[i]);


  int kaos    = (!_flags.Get(MF_CONVEX)) && (!_flags.Get(MF_CLOSED));
  int sconvex =   _flags.Get(MF_CONVEX)  && (!_flags.Get(MF_CLOSED));
  int wireframe = _flags.Get(MF_WIREFRAME);

  if(sconvex)
    for(i=0; i<_nface; i++)
      {
	if(_face[i].N.z > 0)
	  {
	    PE->VAttributes().c = _face[i].c;
	    PE->VAttributes().r = _face[i].r;
	    PE->VAttributes().g = _face[i].g;
	    PE->VAttributes().b = _face[i].b;
	    PE->Reset();

	    for(j=0; j<_face[i].nvertex; j++)
	      PE->Push(&(_face[i].vertex[j]->Projection));

	    if(wireframe)
	      PE->DrawPoly();
	    else
	      PE->FillPoly();
	  }
      }

  for(i=0; i<_nface; i++)
    {
      if(kaos || (_face[i].N.z < 0))
	{
	  PE->VAttributes().c = _face[i].c;
	  PE->VAttributes().r = _face[i].r;
	  PE->VAttributes().g = _face[i].g;
	  PE->VAttributes().b = _face[i].b;
	  PE->Reset();

	  for(j=0; j<_face[i].nvertex; j++)
	    PE->Push(&(_face[i].vertex[j]->Projection));

	  if(wireframe)
	    PE->DrawPoly();
	  else
	    PE->FillPoly();
	}
    }

  PE->PopAttributes();

}

void Mesh::Lighting(void)
{
  int i;

  if(_flags.Get(MF_SMOOTH))
    for(i=0; i<_nvertex; i++)
      {
	int c = _L.x * _vertex[i].N.x +
	        _L.y * _vertex[i].N.y +
                _L.z * _vertex[i].N.z ;

	c = (c < 0) ? 0 : (c >> (22 - D_SHIFT));
	if(c > (63 << D_SHIFT))
	  c = 63 << D_SHIFT;

	if(_flags.Get(MF_COLOR))
	  {
	    _vertex[i].Projection.r = (_vertex[i].r * c) >> (6 + D_SHIFT);
	    _vertex[i].Projection.g = (_vertex[i].g * c) >> (6 + D_SHIFT);
	    _vertex[i].Projection.b = (_vertex[i].b * c) >> (6 + D_SHIFT);
	  }
	else
	  _vertex[i].Projection.c = c;
      }
  else
    for(i=0; i<_nface; i++)
      {
	int c = _L.x * _face[i].N.x +
	        _L.y * _face[i].N.y +
                _L.z * _face[i].N.z ;

	c = (c < 0) ? 0 : (c >> (22 - D_SHIFT));
	if(c > (63 << D_SHIFT))
	  c = 63 << D_SHIFT;

	if(_flags.Get(MF_COLOR))
	  {
	    _face[i].r = (_face[i].or * c) >> (6 + D_SHIFT);
	    _face[i].g = (_face[i].og * c) >> (6 + D_SHIFT);
	    _face[i].b = (_face[i].ob * c) >> (6 + D_SHIFT);
	  }
	else
	  _face[i].c = c;
      }
}

Mesh::~Mesh(void)
{
  if(_resources.Get(MR_FACES))
    delete[] _face;
  if(_resources.Get(MR_VERTICES))
    delete[] _vertex;
}


void Mesh::InvertNormals(void)
{
  int i;
  
  for(i=0; i < _nface; i++)
    {
      _face[i].N.x = -_face[i].N.x;
      _face[i].N.y = -_face[i].N.y;
      _face[i].N.z = -_face[i].N.z;
    }

  if(_resources.Get(MR_SMOOTH))
    for(i=0; i < _nvertex; i++)
      {
	_vertex[i].N.x = -_vertex[i].N.x;
	_vertex[i].N.y = -_vertex[i].N.y;
	_vertex[i].N.z = -_vertex[i].N.z;
      }

}

void Mesh::ComputeNormals(void)
{
  int i; 
  VectorF A,B,N;
  double k;

  for(i=0; i < _nface; i++)
    {
      A.x = _face[i].vertex[0]->x - 
	    _face[i].vertex[1]->x;  
      A.y = _face[i].vertex[0]->y - 
	    _face[i].vertex[1]->y;  
      A.z = _face[i].vertex[0]->z - 
	    _face[i].vertex[1]->z;  
      
      B.x = _face[i].vertex[2]->x - 
	    _face[i].vertex[1]->x;  
      B.y = _face[i].vertex[2]->y - 
	    _face[i].vertex[1]->y;  
      B.z = _face[i].vertex[2]->z - 
	    _face[i].vertex[1]->z;  

      N.x = A.y*B.z - A.z*B.y;
      N.y = A.z*B.x - A.x*B.z;
      N.z = A.x*B.y - A.y*B.x;

      k = N.x * N.x + N.y * N.y + N.z * N.z;
      k = k ? (double)M_BIG / sqrt(k) : 0.0;

      N.x *= k;
      N.y *= k;
      N.z *= k;

      _face[i].N.x = (int)N.x;
      _face[i].N.y = (int)N.y;
      _face[i].N.z = (int)N.z;
    }
}

inline int Mesh::InFace(MFace *F, MVertex *V)
{
  int i;
  for(i=0; i<F->nvertex; i++)
    if(F->vertex[i] == V)
      return 1;
  return 0;
}

void Mesh::Smooth(void)
{
  int i,j;
  for(i=0; i<_nvertex; i++)
    {
      _vertex[i].N.x = 0;
      _vertex[i].N.y = 0;
      _vertex[i].N.z = 0;
    }

  for(i=0; i<_nface; i++)
    for(j=0; j<_face[i].nvertex; j++)
      {
	_face[i].vertex[j]->N.x += _face[i].N.x;
	_face[i].vertex[j]->N.y += _face[i].N.y;
	_face[i].vertex[j]->N.z += _face[i].N.z;
      }

  for(i=0; i<_nvertex; i++)
    _vertex[i].N.Normalize();

  _resources.Set(MR_SMOOTH);
}

void Mesh::Blend(void)
{
  int i;
  int j;

  for(i=0; i<_nvertex; i++)
    {
      _vertex[i].r     = 0;
      _vertex[i].g     = 0;
      _vertex[i].b     = 0;
      _vertex[i].count = 0;
    }

  for(i=0; i<_nface; i++)
    for(j=0; j<_face[i].nvertex; j++)
      {
	_face[i].vertex[j]->r += _face[i].or;
	_face[i].vertex[j]->g += _face[i].og;
	_face[i].vertex[j]->b += _face[i].ob;
	_face[i].vertex[j]->count++;
      }

  for(i=0; i<_nvertex; i++)
    if(_vertex[i].count)
      {
	_vertex[i].r /= _vertex[i].count;
	_vertex[i].g /= _vertex[i].count;
	_vertex[i].b /= _vertex[i].count;
      }
  _resources.Set(MR_BLEND);
}

inline int check_eof(istream& input,  Mesh& M)
{
  if(input.eof())
    {
      M._error_code = ME_EOF;
      return 1;
    }
  return 0;
}

istream& operator >> (istream& input,  Mesh& M)
{

  int dummy;
  VectorF *vertexf;
  int i;
  int j;

  if(!M._resources.Get(MR_VERTICES))
    {

      if(check_eof(input, M))
	return input;

      cerr << "loading object geometry ..." << endl;

      input >> M._nvertex;

      if(!(M._vertex = new MVertex[M._nvertex]))
	{
	  M._error_code = ME_MALLOC;
	  return input;
	}

      M._resources.Set(MR_VERTICES);

      if(check_eof(input, M))
	return input;

      input >> M._nface;
      if(!(M._face = new MFace[M._nface]))
	{
	  M._error_code = ME_MALLOC;
	  return input;
	}

      M._resources.Set(MR_FACES);
      
      input >> dummy;

      if(!(vertexf = new VectorF[M._nvertex]))
	{
	  M._error_code = ME_MALLOC;
	  return input;
	}

      cerr << "loading object ..." << endl;
      cerr << "nvertex= " << M._nvertex << endl;
      cerr << "nface  = " << M._nface   << endl;

      for(i=0; i<M._nvertex; i++)
	{
	  if(check_eof(input, M))
	    return input;
	  input >> vertexf[i].x >> vertexf[i].y >> vertexf[i].z;
	  //      cerr << i << "> [" << vertexf[i].x << ' ' 
	  //                         << vertexf[i].y << ' ' 
	  //                         << vertexf[i].z << ']' << endl;
	}

      for(i=0; i<M._nface; i++)
	{
	  if(check_eof(input, M))
	    return input;
	  
	  input >> M._face[i].nvertex;

	  if(!(M._face[i].vertex = new MVertex*[M._face[i].nvertex]))
	    {
	      M._error_code = ME_MALLOC;
	      return input;
	    }
	  
	  //      cerr << i << '(' << M._face[i].nvertex << ") [";
	  
	  for(j=0; j<M._face[i].nvertex; j++)
	    {
	      int idx;
	      input >> idx;
	      //	  cerr << idx << ' ';
	      M._face[i].vertex[M._face[i].nvertex - j - 1] = 
		                       &(M._vertex[idx - 1]);
	    }

	  //      cerr << "]" << endl;

	}


      cerr << "center and scale ..." << endl;

      VectorF C = {0.0, 0.0, 0.0};
      for(i=0; i<M._nvertex; i++)
	{
	  C.x += vertexf[i].x;
	  C.y += vertexf[i].y;
	  C.z += vertexf[i].z;
	}

      if(M._nvertex)
	{
	  C.x /= (double)M._nvertex;
	  C.y /= (double)M._nvertex;
	  C.z /= (double)M._nvertex;
	}

      cerr << "center = [" << C.x << ' ' << C.y << ' ' << C.z << ']' << endl;
  
      float r = 0,r2;
      
      for(i=0; i<M._nvertex; i++)
	{
	  vertexf[i].x -= C.x;
	  vertexf[i].y -= C.y;
	  vertexf[i].z -= C.z;
	  
	  r2 = vertexf[i].x * vertexf[i].x + 
	       vertexf[i].y * vertexf[i].y + 
	       vertexf[i].z * vertexf[i].z ;
	  
	  r = (r2 > r) ? r2 : r;
	}
      
      r = sqrt(r);

      cerr << "object radius  = " << r << endl;

      double scaling_factor = r ? 10000.0 / r : 0.0;

      cerr << "scaling factor = " << scaling_factor << endl;

      for(i=0; i<M._nvertex; i++)
	{
	  vertexf[i].x *= scaling_factor;
	  vertexf[i].y *= scaling_factor;
	  vertexf[i].z *= scaling_factor;
	}

      cerr << "converting to TAGL++ Mesh" << endl;

      for(i=0; i<M._nvertex; i++)
	{
	  M._vertex[i].x =     (int)vertexf[i].z;
	  M._vertex[i].y =   - (int)vertexf[i].y;
	  M._vertex[i].z =     (int)vertexf[i].x;
	}

      delete[] vertexf;

      cerr << "computing normals ..." << endl;

      M.ComputeNormals();
      
      cerr << "sucessfully loaded." << endl;

    }
  else
    {
      int ncolor;
      int nface;
      ColorF *colorf;
      cerr << "loading object colors ..." << endl;
      
      if(check_eof(input, M))
	return input;

      input >> ncolor >> nface;

      cerr << ncolor << " color entries" << endl;

      if(nface != M._nface)
	{
	  M._error_code = ME_MATCH;
	  return input;
	}

      if(!(colorf = new ColorF[ncolor]))
	{
	  M._error_code = ME_MALLOC;
	  return input;
	}

      for(i=0; i<ncolor; i++)
	{
	  if(check_eof(input, M))
	    return input;
	  input >> colorf[i].r >> colorf[i].g >> colorf[i].b;
	}
      
      for(i=0; i<nface; i++)
	{
	  int idx;

	  if(check_eof(input, M))
	    return input;

	  input >> idx;
	  M._face[i].or = (int)(colorf[idx - 1].r * 256.0);
	  M._face[i].og = (int)(colorf[idx - 1].g * 256.0);
	  M._face[i].ob = (int)(colorf[idx - 1].b * 256.0);
	}

      delete[] colorf;

      cerr << "sucessfully loaded" << endl;
      M._resources.Set(MR_COLORS);
    }

  return input;
}


