/*
 *
 * mesh.h
 * 
 */


#ifndef MESH_H
#define MESH_H


#include "flags.h"
#include "gobj.h"

const Flag MR_NONE     = 0;
const Flag MR_VERTICES = 1;
const Flag MR_FACES    = 2;
const Flag MR_COLORS   = 3;
const Flag MR_SMOOTH   = 4;
const Flag MR_BLEND    = 5;

const Flag ME_NONE   = 0;
const Flag ME_MALLOC = 1;
const Flag ME_EOF    = 2;
const Flag ME_SNTX   = 3;
const Flag ME_MATCH  = 4;

const Flag MF_NONE      = 0;
const Flag MF_CONVEX    = 1;
const Flag MF_CLOSED    = 2;
const Flag MF_SMOOTH    = 3;
const Flag MF_BLEND     = 4;
const Flag MF_COLOR     = 5;
const Flag MF_WIREFRAME = 6;

class MFace
{
 public:
  MFace(void);
  ~MFace(void);

  int        nvertex;
  MVertex  **vertex;
  MVector   N;

  ColorComponent  r,  g,  b;
  ColorComponent or, og, ob;
  ColorCode      c;
};

class Mesh : public GraphicObject
{
 public:
  Mesh(void);
  virtual ~Mesh(void);
  virtual void RotX(Angle r);
  virtual void RotY(Angle r);
  virtual void RotZ(Angle r);
  virtual void Translate(int tx, int ty, int tz);
  virtual void Scale(double sx, double sy, double sz);
  virtual void Draw(PolygonEngine *PE);
  virtual void Lighting(void);

  Flag   ErrorCode(void);
  Flags& Mode(void);

  void Setup(PolygonEngine *PE);

  void Smooth(void);
  void Blend(void);

  void InvertNormals(void);

 protected:

  int InFace(MFace *F, MVertex *V);

  void ComputeNormals(void);

  MVertex *_vertex;
  int      _nvertex;
  MFace   *_face;
  int      _nface;

  Flags _resources;
  Flags _flags;
  Flag  _error_code;

  friend int check_eof(istream& input,  Mesh& M);
  friend istream& operator >> (istream& input,  Mesh& M);
  friend ostream& operator << (ostream& output, Mesh& M);
};


istream& operator >> (istream& input,  Mesh& M);
ostream& operator << (ostream& output, Mesh& M);


#include "mesh_i.h"

#endif
