#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include"ecran256.h"
#include"mouse.h"

#define RANDOM(x) (random() % (x))
#define randomize() srandom(time(NULL))

/* Definition de la montagne:*/
#define ntaille 7
#define taille 128  /*taille = 2^ntaille*/
#define irregularite 2

/* Stocke la montagne et la couleur associee a chaque sommet:*/
byte tab_h[taille+1][taille+1], tab_c[taille+1][taille+1];

#define w_screen 320
#define h_screen 200

#define dec 100         /* Distance oeil/ecran              */
#define d_max 20        /* Distance de vision maximale      */
#define d_min 1         /* Distance de vision minimale      */
#define d_obscurite 16  /* Distance ou commence l'obscurite */
#define obscurite 4
    /* on doit avoir (d_max-d_obscurite)*2^obscurite <= 64  */
#define cote 6          /* Distance entre 2 sommets         */

#define p_heigth 10    /* Hauteur du point de vue          */
#define h_lac 50        /* Niveau de l'eau                  */

/*Numeros des palettes: */
#define VENUS      1
#define TERRE      2
#define MERCURE    3
#define BROUILLARD 4
#define LSD        5
#define LUNE       6
#define TERRILS    7
#define MARS       8
#define BANQUISES  9

#define TESTS

#define Fixed unsigned int
#define E(x) ((x) >> 8)            /* partie entiere   */
#define fl(x) ((x) & 255)          /* partie flottante */
#define b2F(b) ((Fixed)(b) << 8)   /* byte -> Fixed    */
#define f2F(f) (Fixed)((f) * 256)  /* float -> Fixed   */

#define acces_h(u,v) (int)tab_h[((u) & (taille - 1))][((v) & (taille - 1))]
#define acces_c(u,v) (int)tab_c[((u) & (taille - 1))][((v) & (taille - 1))]

#define int short

/* Tables cosinus/sinus*/
Fixed cs[256], sn[256];
/* Direction de depart du point de vue: */
byte dirZ = 0;
/* et le cosinus et le sinus:*/
Fixed xdirZ, ydirZ;

byte dirY = 0;
Fixed xdirY, zdirY;
#define delta_dirY 63
/* Horizon: */
byte horizon[255];

/* Position du point de vue: */
Fixed up = 0, vp = 0;
Fixed hp = b2F(128);
long vitessep = 1;

/*Flags divers: */
byte shading_x = 1,shading_y = 1, lissage = 1,
     map = 0, assembler = 0, etoiles = 1, points = 0, vagues = 1;

/* Positions des etoiles: */
#define nb_etoiles 250
byte x_etoiles[nb_etoiles],y_etoiles[nb_etoiles];

/* Numero de l'image courante; permet d'animer les vagues*/
byte no_image = 0;

/* Initialisation d'un sommet de la montagne*/
void init_sommet(int i,int j,byte altmoy,int p_taille)
 {
  int h_int;
  byte cc;

  if (i<taille && j<taille)
  {
   if (tab_h[i][j]==0) /* nouveau sommet*/
   {
    h_int= (int)altmoy+(RANDOM(p_taille*2)-(p_taille))*irregularite;
    /* Reste dans l'intervalle [1..(255-hauteur du point de vue)]:*/
    if (h_int<1)
     tab_h[i][j]=1;
    else
     if (h_int > (255-p_heigth))
      tab_h[i][j]=255-p_heigth;
     else
      tab_h[i][j]=(byte)h_int;

    move(i,j);
    cc = tab_h[i][j];
    cc = (cc <= h_lac) ? 127 : cc >> 2;
    put_pixel(cc);
   }
  }
 }

/* Initialise un carre de montagne recursivement: */
void init_mont(int i1,int j1,int i2,int j2,int p_taille)
 {
   int altmoy;
   int nouvtaille;


   nouvtaille = p_taille >> 1;

   altmoy = ((int)acces_h(i1,j1) + (int)acces_h(i1+p_taille,j1)) >> 1;
   init_sommet(i1+nouvtaille,j1,altmoy,p_taille);

   altmoy = ((int)acces_h(i1,j1) + (int)acces_h(i1,j1+p_taille)) >> 1;
   init_sommet(i1,j1+nouvtaille,altmoy,p_taille);

   altmoy = ((int)acces_h(i1,j1+p_taille) + (int)acces_h(i1+p_taille,j1+p_taille)) >> 1;
   init_sommet(i1+nouvtaille,j1+p_taille,altmoy,p_taille);

   altmoy = ((int)acces_h(i1+p_taille,j1) + (int)acces_h(i1+p_taille,j1+p_taille)) >> 1;
   init_sommet(i1+p_taille,j1+nouvtaille,altmoy,p_taille);

   altmoy=((int)acces_h(i1,j1)+(int)acces_h(i2,j1)+(int)acces_h(i1,j2)+
	     (int)acces_h(i2,j2)) >> 2;
   init_sommet(i1+nouvtaille,j1+nouvtaille,(byte) altmoy,p_taille);
   if (p_taille>0)
    {
     init_mont(i1,j1,i1+nouvtaille,j1+nouvtaille,nouvtaille);
     init_mont(i1+nouvtaille,j1,i2,j1+nouvtaille,nouvtaille);
     init_mont(i1,j1+nouvtaille,i1+nouvtaille,j2,nouvtaille);
     init_mont(i1+nouvtaille,j1+nouvtaille,i2,j2,nouvtaille);
    }
 }

void init_mountain_fractal()
{
  int i,j;

  for(i=0; i<taille; i++)
   for(j=0; j<taille; j++)
    tab_h[i][j]=0;

  init_sommet(0,0,0,taille);

  init_mont(0,0,taille,taille,taille);

  for(i=0; i<taille; i++)
    for(j=0; j<taille; j++)
      if (tab_h[i][j] < h_lac)
       tab_h[i][j] = h_lac;

  for(i=0; i<taille+1; i++) tab_h[i][taille] = tab_h[i][0];
  for(i=0; i<taille+1; i++) tab_h[taille][i] = tab_h[0][i];
  tab_h[taille][taille] = tab_h[0][0];
 }

/* Initialisation des couleurs pour le lissage: */
void init_colors()
{int i,j;
 int deniv;

 for(i = 0; i < taille; i++)
 {
  for(j = 0; j < taille; j++)
   {
    if (acces_h(i,j)==h_lac)
     if (acces_h(i+1,j)   == h_lac &&
	 acces_h(i,j+1)   == h_lac &&
	 acces_h(i-1,j)   == h_lac &&
	 acces_h(i,j-1)   == h_lac &&
	 acces_h(i+1,j+1) == h_lac &&
	 acces_h(i-1,j+1) == h_lac &&
	 acces_h(i-1,j+1) == h_lac &&
	 acces_h(i-1,j-1) == h_lac) /* Un point dans le lac */
      tab_c[i][j] = 127 - RANDOM(32);
	/*Pourquoi RANDOM(32) et pas RANDOM(63) ? Question d'esthetique...*/
     else
      tab_c[i][j] = 64;                /* Un point sur le bord du lac */
    else
    {
     if (acces_h(i+1,j)==h_lac ||   /* Un point sur le tour du lac */
	 acces_h(i,j+1)==h_lac ||
	 acces_h(i-1,j)==h_lac ||
	 acces_h(i,j-1)==h_lac)
     tab_c[i][j] = 64;
     else
     {                              /* Un point de la montagne */
      deniv=acces_h(i-1,j)  +
	   acces_h(i,j+1)   +
	   acces_h(i-1,j+1) +
	   acces_h(i-1,j-1) -

	   acces_h(i+1,j)   -
	   acces_h(i,j-1)   -
	   acces_h(i+1,j-1) -
	   acces_h(i+1,j+1);

      deniv = (deniv+32);

      if (deniv>63)
       deniv = 63;
      if (deniv<1)
       deniv = 1;
      tab_c[i][j] = deniv;
     }
    }
   }
 }
  for(i=0; i<taille+1; i++) tab_c[i][taille] = tab_c[i][0];
  for(i=0; i<taille+1; i++) tab_c[taille][i] = tab_c[0][i];
  tab_c[taille][taille] = tab_c[0][0];
}

typedef struct 
 {
  byte r,v,b;
 } coul;

void init_palette(byte planete)
{
 coul ciel, montagne, lac, etoiles;
 byte i,c;

 switch(planete)
 {
 case VENUS:
 {
  ciel.r     = 0;  ciel.v     = 0;   ciel.b     = 0;
  montagne.r = 63; montagne.v = 63;  montagne.b = 0;
  lac.r      = 63; lac.v      = 63;  lac.b      = 63;
  etoiles.r  = 63; etoiles.v  = 63;  etoiles.b  = 0;
  break;
 }
 case TERRE:
 {
  ciel.r     = 0;  ciel.v     = 0;   ciel.b     = 10;
  montagne.r = 0;  montagne.v = 63;  montagne.b = 0;
  lac.r      = 0;  lac.v      = 0;   lac.b      = 63;
  etoiles.r  = 63; etoiles.v  = 63;  etoiles.b  = 0;
  break;
 }
 case MERCURE:
 {
  ciel.r     = 10; ciel.v     = 0;   ciel.b     = 0;
  montagne.r = 63; montagne.v = 63;  montagne.b = 63;
  lac.r      = 63; lac.v      = 63;  lac.b      = 0;
  etoiles.r  = 63; etoiles.v  = 63;  etoiles.b  = 0;
  break;
 }
 case BROUILLARD:
 {
  ciel.r     = 50; ciel.v     = 50; ciel.b      = 50;
  montagne.r = 0;  montagne.v = 0;  montagne.b  = 0;
  lac.r      = 0;  lac.v      = 0;  lac.b       = 30;
  etoiles.r  = 50; etoiles.v  = 50; etoiles.b   = 50;
  break;
 }
 case LSD:
 {
  ciel.r     = 63; ciel.v     = 0;  ciel.b      = 63;
  montagne.r = 0;  montagne.v = 63; montagne.b  = 63;
  lac.r      = 0;  lac.v      = 63; lac.b       = 0;
  etoiles.r  = 63; etoiles.v  = 0;  etoiles.b   = 0;
  break;
 }
 case LUNE:
 {
  ciel.r     = 0;  ciel.v     = 0;  ciel.b      = 0;
  montagne.r = 63; montagne.v = 63; montagne.b  = 63;
  lac.r      = 30; lac.v      = 30; lac.b       = 30;
  etoiles.r  = 63; etoiles.v  = 63; etoiles.b   = 63;
  break;
 }
 case TERRILS:
 {
  ciel.r     = 0;  ciel.v     = 0;  ciel.b      = 10;
  montagne.r = 63; montagne.v = 30; montagne.b  = 10;
  lac.r      = 10; lac.v      = 30; lac.b       = 0;
  etoiles.r  = 63; etoiles.v  = 63; etoiles.b   = 0;
  break;
 }
 case MARS:
 {
  ciel.r     = 10;  ciel.v     = 0;  ciel.b      = 0;
  montagne.r = 50; montagne.v = 15; montagne.b  = 0;
  lac.r      = 63; lac.v      = 40; lac.b       = 0;
  etoiles.r  = 0; etoiles.v  = 0; etoiles.b   = 30;
  break;
 }
 case BANQUISES:
 {
  ciel.r     = 30;  ciel.v     = 30;  ciel.b      = 63;
  montagne.r = 63;  montagne.v = 63;  montagne.b  = 63;
  lac.r      = 0;   lac.v      = 0;   lac.b       = 30;
  etoiles.r  = 30;  etoiles.v  = 30;  etoiles.b   = 63;
  break;
 }
 }

 setcolor256(0,0,0,0);
 for(i = 1; i < 64; i++)
 {
  setcolor256(i,(byte)(montagne.r + (float)(ciel.r - montagne.r) * i / 64),
		(byte)(montagne.v + (float)(ciel.v - montagne.v) * i / 64),
		(byte)(montagne.b + (float)(ciel.b - montagne.b) * i / 64));
  move(i,0);
  put_pixel(i);
 }

 for(i = 0; i < 64; i++)
 {
  setcolor256(64+i,(byte)(ciel.r + (float)(lac.r - ciel.r) * i / 64),
		   (byte)(ciel.v + (float)(lac.v - ciel.v) * i / 64),
		   (byte)(ciel.b + (float)(lac.b - ciel.b) * i / 64));
  move(i+64,0);
  c = i + 64;
  put_pixel(c);
 }

 setcolor256(128,63,0,0);
 setcolor256(129,etoiles.r,etoiles.v,etoiles.b);
}

#define draw_assembler       \
      asm {                  \
      push es;               \
      push di;               \
      push ax;               \
      push bx;               \
      push cx;               \
			     \
      mov es,segment_ecran;  \
      mov di,offset_pixel;   \
      mov bl,byte ptr (pc);  \
      xor bh,bh;             \
      mov ax, (dc);          \
      mov bh,bl;             \
      xor bl,bl;             \
      mov cx,word ptr (dy);  \
      }                      \
      boucle:                \
      asm {                  \
      mov [es:di],bh;        \
      add bx,ax;             \
      add di,80;             \
      loop boucle:;          \
			     \
      pop cx;                \
      pop bx;                \
      pop ax;                \
      pop di;                \
      pop es;                \
      }

#define draw_c              \
     for(; (yc<ymax); yc++) \
     {                      \
      c = cc >> 8;          \
      put_pixel(c);         \
      inc_y;                \
      cc += dc;             \
     }

#define draw_c_noshading    \
    for(; (yc<ymax); yc++)  \
    {                       \
     put_pixel(ppc);        \
     inc_y;                 \
    }

Fixed   tf[d_max+1];
Fixed  tdu[d_max+1];
Fixed tpas[d_max+1];

void pre_calcul()
{
 byte i;

 for(i = d_min; i <= d_max; i++)
  {
   tf[i]    = f2F((float)dec / ((float)i * cote));
   tdu[i]   = f2F((float)((unsigned long)i << 7) / dec);
      /* i << 7 = i * 128 = i * (width/2)*/
   tpas[i]  = f2F((float)i/dec);
  }
}


void show_mountain()
{
 byte dv;
 byte i;
 byte hor;
 Fixed b,d;
/* Fixed A;*/
 long unsigned A;
 int B;
 Fixed * _tf;
 Fixed * _tdu;
 Fixed * _tpas;
/* Tableaux contenant les ordonnees a l'ecran (Y) et les couleurs
  de la tranche courante:*/
 byte tph[255], tpc[255];
 byte *_tph, *_tpc;
 byte Eup, Evp;
 byte *_tab_h;
 byte h00, h01, h10, h11;
 byte decalh;
 Fixed h1, ph;
 byte *_horizon;


/** Initialisation du niveau de l'horizon: *********************************/
 for(i=0; i<255; i++) horizon[i]=199;

/** Calcul la hauteur a la position de l'observateur: **********************/
  Eup = E(up);
  Eup &= (taille-1);
  Evp = E(vp);
  Evp &= (taille-1);

  /* Interpolation en divisant le carre en deux triangles: */
  _tab_h = &tab_h[Eup][Evp];
#define vague(h) if ((h) == h_lac) (h) += (decalh & 3)


  if (fl(up) > fl(vp))
  {
  /* Accede aux 3 hauteurs :*/
   h00 = *_tab_h;
   _tab_h += taille+2;
   h11 = *_tab_h;
   _tab_h--;
   h10 = *_tab_h;

  /* Change les hauteurs des vagues: */
  if (vagues)
  {
   decalh  = Eup;
   decalh += Evp;
   decalh += no_image;
   vague(h00);
   decalh++;
   vague(h10);
   decalh++;
   vague(h11);
  }

   h1  = h10;
   h1 -= h00;
   h1 *= fl(up);

   ph  = h11;
   ph -= h10;
   ph *= fl(vp);

   ph += h1;
   ph += b2F(h00);
  }
  else
  {
  /* Accede aux 3 hauteurs :*/
   h00 = *_tab_h;
   _tab_h++;
   h01 = *_tab_h;
   _tab_h += taille+1;
   h11 = *_tab_h;

  /* Change les hauteurs des vagues: */
  if (vagues)
  {
   decalh  = Eup;
   decalh += Evp;
   decalh += no_image;
   vague(h00);
   decalh++;
   vague(h01);
   decalh++;
   vague(h11);
  }

   h1  = h01;
   h1 -= h00;
   h1 *= fl(vp);

   ph  = h11;
   ph -= h01;
   ph *= fl(up);

   ph += h1;
   ph += b2F(h00);
   ph += b2F(p_heigth);
  }

/** Nivelle la hauteur du point de vue: ***********************************/
 if (ph > hp) hp = ph;

/** Coeffs pour le calcul de y: *******************************************/
 A = (dec * (long unsigned)hp) / cote;
 B = ((int)zdirY * (int)dec)/ (int)xdirY;

/** Rotation de du et dv autour de l'axe Z: *******************************/
 /* il ne faut pas '>> 8' car d_min n'a pas de partie flottante*/
 b = ((long)(int)ydirZ * (long)d_min);
 d = ((long)(int)xdirZ * (long)d_min);


 /* Initialisations des pointeurs sur les elements des tableaux*/
 _tf   =   &tf[d_min];
 _tdu  =  &tdu[d_min];
 _tpas = &tpas[d_min];

 for(dv = d_min;dv <= d_max; dv++)
  {
/**************************************************************************/
/* Construit le tableau des ordonnees (a l'ecran) de la tranche courante: */
/* void construct_tables(byte dv)                                         */
{
 Fixed u, v, f, du, pas, pas_u, pas_v;
 Fixed y;

 Fixed ph, h1;
 byte h00, h01, h10, h11;

 Fixed pc, c1;
 byte c00, c01, c10, c11;

 byte delta_c, lim1, lim2;

 Fixed a, c;

 Fixed cc;

 byte Eu, Ev, decalh;

 byte *_tab_h, *_tab_c, *_horizon;


 f   = *_tf;    /* Hauteur de reference a la distance dv*/
 du  = *_tdu;   /* Decalage sur u pour la premiere colonne de l'ecran*/
 pas = *_tpas;  /* Pas sur l'axe u*/
 y   = (h_screen >> 1) + (Fixed)(A / ((long unsigned)dv << 8)) + B;
		/* Ordonnee a l'ecran pour un sommet de hauteur 0*/

 /* Incremente pour la prochaine fois: */
 _tf++; _tdu++; _tpas++;

 /* Rotation de du et dv:*/
 a = ((long)(int)xdirZ * (long)du) >> 8;
 c = ((long)(int)ydirZ * (long)du) >> 8;

 /*Calcul u et v de depart (qui correspondent a la colonne de gauche)*/
 /* obtenus par: (u,v) = (up,vp) + R(dirZ) * (du,dv)                 */
 u  = up;  u -= a;  u -= b;
 v  = vp;  v -= c;  v += d;

 /* Incremente pour la prochaine fois: */
 b += ydirZ;
 d += xdirZ;

 /* Rotation de pas par R(dirZ):      */
 /* (pas_u,pas_v) = R(dirZ) * (pas,0) */
 pas_u = ((long)(int)xdirZ * (long)pas) >> 8;
 pas_v = ((long)(int)ydirZ * (long)pas) >> 8;

 /* Gere l'obscurite: */
 if (dv > d_obscurite)
 {
  delta_c = (dv - d_obscurite) << obscurite;
  lim1 = (64 - delta_c);
  lim2 = (64 + delta_c);
 }

 _tph = &tph[0];
 _tpc = &tpc[0];
 _horizon = &horizon[0];

 for(i = 0; i < 255; i++) /* Pour chaque colonne de l'ecran*/
 {
  Eu = E(u);
  Eu &= (taille-1);
  Ev = E(v);
  Ev &= (taille-1);

  /* Interpolation en divisant le carre en deux triangles: */
  _tab_h = &tab_h[Eu][Ev];

  if (fl(u) > fl(v))
  {
  /* Accede aux 3 hauteurs :*/
   h00 = *_tab_h;
   _tab_h += taille+2;
   h11 = *_tab_h;
   _tab_h--;
   h10 = *_tab_h;

  /* Change les hauteurs des vagues: */
  if (vagues)
  {
   decalh  = Eu;
   decalh += Ev;
   decalh += no_image;
   vague(h00);
   decalh++;
   vague(h10);
   decalh++;
   vague(h11);
  }

   h1  = h10;
   h1 -= h00;
   h1 *= fl(u);

   ph  = h11;
   ph -= h10;
   ph *= fl(v);

   ph += h1;
   ph += b2F(h00);
  }
  else
  {
  /* Accede aux 3 hauteurs :*/
   h00 = *_tab_h;
   _tab_h++;
   h01 = *_tab_h;
   _tab_h += taille+1;
   h11 = *_tab_h;

  /* Change les hauteurs des vagues: */
  if (vagues)
  {
   decalh  = Eu;
   decalh += Ev;
   decalh += no_image;
   vague(h00);
   decalh++;
   vague(h01);
   decalh++;
   vague(h11);
  }

   h1  = h01;
   h1 -= h00;
   h1 *= fl(v);

   ph  = h11;
   ph -= h01;
   ph *= fl(u);

   ph += h1;
   ph += b2F(h00);
  }

  /* Perspective sur la hauteur: */
  ph =  (Fixed) (((long unsigned)ph * (long unsigned)f) >> 16);
  ph = y - ph;

  /* assure d'atteindre le bas de l'ecran: */
  if (dv == d_min && (int)ph<199) ph = 199;
  else
  /* met ph dans l'intervalle d'un byte: */
   if ((int)ph < 0) ph = 0;
   else if (ph > 255) ph = 255;

#define fondu(c) if ((c) < lim1) (c) += delta_c;                    \
		 else if ((c) > lim2) (c) -= delta_c; else (c) = 64;

  if (shading_x)
  {
   _tab_c = &tab_c[Eu][Ev];
   if (fl(u) > fl(v))
   {
   /* Accede aux 3 couleurs :*/
    c00 = *_tab_c;
    _tab_c += taille+2;
    c11 = *_tab_c;
    _tab_c--;
    c10 = *_tab_c;

    /* Change les couleurs des zones dans l'obscurite: */
    if (dv > d_obscurite)
    {
     fondu(c00);
     fondu(c10);
     fondu(c11);
    }

    c1  = c10;
    c1 -= c00;
    c1 *= fl(u);

    pc  = c11;
    pc -= c10;
    pc *= fl(v);

    pc += c1;
    pc += b2F(c00);
   }
   else
   {
   /* Accede aux 3 couleurs :*/
    c00 = *_tab_c;
    _tab_c++;
    c01 = *_tab_c;
    _tab_c += taille+1;
    c11 = *_tab_c;

    /* Change les couleurs des zones dans l'obscurite: */
    if (dv > d_obscurite)
    {
     fondu(c00);
     fondu(c01);
     fondu(c11);
   }

   c1  = c01;
   c1 -= c00;
   c1 *= fl(v);

   pc  = c11;
   pc -= c01;
   pc *= fl(u);

   pc += c1;
   pc += b2F(c00);
  }

   cc  = pc;
   pc >>= 8;
  }
  else
  /* Pas de shading en X: */
  {
   cc = acces_c(E(u), E(v));
   pc = cc;
   cc <<= 8;
  }

  /* Incremente (u,v) pour la prochaine colonne: */
  u += pas_u;
  v += pas_v;

  /* Si la tranche n'est pas la premiere qu'on calcule,
    on affiche les points entre cette tranche et la tranche precedente.*/
  if (dv > d_min)
  {
   byte c2, ppc;
   byte yc, ymax;
   byte hor;
   int  dy, dc;

   ymax = *_tph;
   yc   = ph;
   hor  = *_horizon;
   if ((yc < ymax) /* trace uniquement les segments qui montent*/
	&& (yc < hor)) /* et ceux qui sont au dessus de l'horizon*/
   {
    *_horizon = yc;
    if (shading_y)
    {
     c2 = *_tpc;
     /* distance entre les 2 points de l'ecran: */
     dy = ymax - yc;
     /* pas du shading: */
     dc =(((long)c2-(long)pc) << 8) / dy;
     /* Le pas pour le lissage de Gouraud est calcule.
      On peut clipper le segment. Ca se resume a: */
     if (ymax > hor)
      {
       ymax = hor;
       dy = ymax - yc;
      }

     /* Place le pointeur sur le premier point: */
     move(i,yc);

     /* Affiche un point ...      */
     if (points) { put_pixel(c2); }
     else
     /* ... ou un bout de colonne:*/
     { if (assembler) { /* draw_assembler */ } else { draw_c } }
    }

    else
    /* Pas de shading en Y: */
    {
     ppc = pc;
     if (ymax > hor) ymax = hor;
     move(i,yc);
     /* Affiche un point ...      */
     if (points) { put_pixel(ppc); }
     /* ... ou un bout de colonne:*/
     else { draw_c_noshading; }
    } /* if (yshading) ... */

   } /* if ((yc < ymax) ... */
  } /* if (dv > d_min) ... */

  *_tph = ph;
  *_tpc = pc;

  _tph++;
  _tpc++;
  _horizon++;

 } /* for(i = 0; ... */
}
/* fin construct_tables(byte dv)                                        */
/************************************************************************/

  }



 /*Efface le ciel*/
 _horizon = &horizon[0];
 for(i=0; i<255; i++)
 {
  hor = *_horizon;
  if (hor>0)
  {
   move(i,0);
   for(; hor > 0; hor--)
   {
    put_pixel(63);
    inc_y;
   }
  }
  _horizon++;
 }
}

void show_mountain_dessus()
{byte i,j,cc;

 for(i = 0; i < taille; i++)
 {
  move(i,taille);
  for(j = 0; j < taille; j++)
   {
     cc = tab_h[i][j];
     cc = (cc == h_lac) ? tab_c[i][j] : cc >> 2;
     put_pixel(cc);
     dec_y;
   }
 }
 move(E(up) & (taille - 1),(taille - E(vp)) & (taille - 1));
 put_pixel(128);
}

void init_etoiles()
{
 byte i;

 for(i=0; i<nb_etoiles; i++)
  {
   x_etoiles[i] = RANDOM(254);
   y_etoiles[i] = RANDOM(254);
  }
}

void aff_etoiles()
{
 byte i, x, y;

 for(i=0; i<nb_etoiles; i++)
  {
   x = x_etoiles[i] + dirZ;
   if (x < 255)
   {
    y = y_etoiles[i] + dirY;
    if (horizon[x] > y)
    {
     move(x,y);
     put_pixel(129);
    }
   }
  }
}

void init_cos_sin()
{
 int i;

 for(i=0; i<256; i++)
 {
  cs[i]=f2F(cos((float)i/256*2*3.14159));
  sn[i]=f2F(sin((float)i/256*2*3.14159));
 }
}

void initalisations()
{
 randomize();

 init256();
 set_page(1); show_page(1);

 init_palette(VENUS);
 init_mountain_fractal();
 init_colors();
 init_etoiles();

 pre_calcul();

 init_cos_sin();

 xdirZ = cs[dirZ];
 ydirZ = sn[dirZ];

 xdirY = cs[dirY];
 zdirY = sn[dirY];
}

main()
{
 char cc;
 byte nopage=0;
 int du,dv;
 int b, d;

  initalisations();

  cc=' ';
  while (cc!='q')
  {
   int buttons;
   set_page(nopage);
   if (points) clr_page;
   show_mountain();
   if (etoiles) aff_etoiles();
   if (map) show_mountain_dessus();
   show_page(nopage);
   nopage=1 - nopage;

   no_image++;

   buttons = deplac_mouse(&du,&dv);

   if(buttons & 1)
     vitessep += 10;
   if(buttons & 4)
     vitessep -= 10;
   if((buttons & 2) || (buttons & 1) && (buttons & 4))
     vitessep = 0;

   du >>= 3;
   dirZ -= du;
   xdirZ = cs[dirZ];
   ydirZ = sn[dirZ];

   dv >>= 3;
   dirY += dv;

   if((dirY > 63) && (dirY < 128))
     dirY = 63;

   if((dirY > 128) && (dirY < 193))
     dirY = 193;

   xdirY = cs[dirY];
   zdirY = sn[dirY];

   b = ((int)ydirZ * vitessep) >> 8;
   b = ((long)b * (int)xdirY) >> 8;

   d = ((int)xdirZ * vitessep) >> 8;
   d = ((long)d * (int)xdirY) >> 8;

   up = up - b;
   vp = vp + d;

   hp = hp + (((int)zdirY*vitessep) >> 8);

   if ((cc = readkbd()))
   {
    switch(cc)
     {
      case 'x':shading_x = 1 - shading_x; break;
      case 'y':shading_y = 1 - shading_y; break;
      case 'v':vagues = 1 - vagues; break;
      case 'l':lissage = 1 - lissage; break;
      case 'm':map = 1 - map; break;
      case 'a':printf("assembler version not available under Unix\n"); break;
      case 'e':etoiles = 1 - etoiles; break;
      case 'p':points = 1 - points; break;
      case '1':init_palette(VENUS); break;
      case '2':init_palette(TERRE); break;
      case '3':init_palette(MERCURE); break;
      case '4':init_palette(BROUILLARD); break;
      case '5':init_palette(LSD); break;
      case '6':init_palette(TERRILS); break;
      case '7':init_palette(LUNE); break;
      case '8':init_palette(MARS); break;
      case '9':init_palette(BANQUISES); break;
/*      case '4':up+=256; break;
      case '6':up-=256; break;
      case '8':vp+=256; break;
      case '5':vp-=256; break;
      case '7':
      {
       dirZ += 1;
       xdirZ = cs[dirZ];
       ydirZ = sn[dirZ];
       break;
      }
      case '9':
      {
       dirZ -= 1;
       xdirZ = cs[dirZ];
       ydirZ = sn[dirZ];
       break;
      } */
      case '+':
      {
       vitessep += 10;
       break;
      }
      case '-':
      {
       vitessep -= 10;
       break;
      }
      case '0':
      {
       vitessep = 0;
       break;
      }
      case 'r':
      {
       randomize();
       init_mountain_fractal();
       init_colors();
       break;
      }

      case 'i':
      {
/*       fin256();
         clrscr(); */
       printf("Shading x : %u\n",shading_x);
       printf("Shading y : %u\n",shading_y);
       printf("Lissage   : %u\n",lissage);
       printf("Assembleur: %u\n",assembler);
/*       getch();
         init256();
         init_palette(VENUS); */
       break;
      }
     }
   }
  }
 fin256();
 return 0;
}
