/*
 *
 * TAGL driver for povray
 *
 */

#include "gport.h"

extern "C"
{
void display_finished(void);
void display_init(int width, int height);
void display_close(void);
void display_plot(int x, int y, unsigned char R,
				unsigned char G,
				unsigned char B);

void unix_init_POVRAY(void);
}

#include <iostream.h>
#include <stdlib.h>


int D8[64] =
   {  0, 32,  8, 40,  2, 34, 10, 42 ,
     48, 16, 56, 24, 50, 18, 58, 26 ,
     12, 44,  4, 36, 14, 46,  6, 38 ,
     60, 28, 52, 20, 62, 30, 54, 22 ,
      3, 35, 11, 43,  1, 33,  9, 41 ,
     51, 19, 59, 27, 49, 17, 57, 25 ,
     15, 47,  7, 39, 13, 45,  5, 37 ,
     63, 31, 55, 23, 61, 29, 53, 21 };


GraphicPort *GP;

static int _direct = 0;

void unix_init_POVRAY(void)
{
}

void display_finished()
{
}

void display_init(int width, int height)
{
  GP = MakeGraphicPort("POVRAY", width, height, MSG_NONE);
  if(!GP)
     {
       cerr << "could not open graphic port\n";
       exit(-1);
     }
  GP->RGBMode();
  _direct = GP->SingleBuffer();
}

void display_close(void)
{
   char q = 0;

   if(!_direct)
      GP->SwapBuffers();

   while(q != 'q')
      {
	while(!(q = GP->GetKey()));
	if(!_direct)
           GP->SwapBuffers();
      }


   delete GP;
}

void display_plot(int x, int y, 
                  unsigned char R, unsigned char G, unsigned char B)
{
  int r = R;
  int g = G;
  int b = B;
  int rerr, gerr, berr;

  int treshold = D8[((y & 7) << 3) + (x & 7)];

  unsigned char *gptr = GP->GraphMem() + GP->BytesPerLine() * y + 
                        GP->BytesPerPixel() * x;

  unsigned long colorcode;


  switch(GP->BytesPerPixel())
      {
         case 1:

	   rerr = r & 63;
           gerr = g & 63;
           berr = b & 63;

	   r = r >> 6;
	   g = g >> 6;
	   b = b >> 6;


	   if((rerr > treshold) && (r < 3))
		r++;

	   if((gerr > treshold) && (g < 3))
		g++;

	   if((berr > treshold) && (b < 3))
		b++;


           colorcode = b | (g << 2) | (r << 4);
           *gptr = GP->Colormap()[colorcode];
           break;

         case 2:

	   rerr = (r & 7) << 3;
	   gerr = (g & 7) << 3;
	   berr = (b & 7) << 3;

	   r = r >> 3;
	   g = g >> 3;
	   b = b >> 3;
	

	   if((rerr > treshold) && (r < 31))
		r++;

	   if((gerr > treshold) && (g < 31))
		g++;

	   if((berr > treshold) && (b < 31))
		b++;

           colorcode = b | (g << 5) | (r << 10);
           *(unsigned short *)gptr = (unsigned short)colorcode;           
           break;

         case 3:
           *(gptr++) = (char)b;
           *(gptr++) = (char)g;
           *gptr     = (char)r;
           break;
      }
  if(!_direct && !x && !(y % 10))	
     GP->SwapBuffers();

}
