
/*
 *
 * interface between tagl and wt.
 * allows both X11 and svga graphics.
 *
 */



#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <math.h>

extern "C" {
#include "wt.h"
#include "error.h"
#include "input.h"
#include "x11graphics.h"
}

#include <gport.h>


extern GraphicPort *GP;

static void add_special(Intent *intent, int special);

static Boolean rotating_cw = False;
static Boolean rotating_ccw = False;
static Boolean moving_forward = False;
static Boolean moving_backward = False;
static Boolean running = False;
static Boolean strafing = False;


void init_input_devices(void)
{
}


void end_input_devices(void)
{
}

/*
Intent *read_input_devices(void)
{
  static Intent intent;

  intent.force_x = intent.force_y = intent.force_z = 0.0;
  intent.force_rotate = 0.0;
  intent.n_special = 0;
  
  switch (GP->GetKey()) 
    {
    case 'j':
      rotating_ccw = True;
      break;
      
    case 'k':
      rotating_cw = True;
      break;
      
    case 'i':
      moving_forward = True;
      break;
      
    case ',':
      moving_backward = True;
      break;
      
    case '<':
      running = True;
      break;
      
    case 'w':
      strafing = True;
      break;
      
    case ' ':
      add_special(&intent, INTENT_JUMP);
      break;
      
    case 'q':
      add_special(&intent, INTENT_END_GAME);
      break;

    case '1':
      add_special(&intent, INTENT_ACTION1);
      break;

    case '2':
      add_special(&intent, INTENT_ACTION2);
      break;

    case '3':
      add_special(&intent, INTENT_ACTION3);
      break;

    case '4':
      add_special(&intent, INTENT_ACTION4);
      break;

    case '5':
      add_special(&intent, INTENT_ACTION5);
      break;

    default:
      break;
    }

  if (rotating_cw) 
    {
      if (strafing)
	intent.force_y -= MOVE_FORCE;
      else
	intent.force_rotate -= TURN_FORCE;
    }
  if (rotating_ccw) 
    {
      if (strafing) 
	{
	  intent.force_y += MOVE_FORCE;
	} 
      else
	intent.force_rotate += TURN_FORCE;
    }
  if (moving_forward)
    intent.force_x += MOVE_FORCE;
  if (moving_backward)
    intent.force_x -= MOVE_FORCE;
  if (running) 
    {
      intent.force_x *= 2.0;
      intent.force_y *= 2.0;
      intent.force_z *= 2.0;
      intent.force_rotate *= 2.0;
    }
  
  return &intent;
}
*/

#define MOUSE_XRANGE GP->Width()
#define MOUSE_YRANGE GP->Height()

Intent *read_input_devices(void)
{
     static Intent intent;
     int mouse_x, mouse_y, mouse_button;


     intent.n_special = 0;
     intent.force_y = 0.0;
     intent.force_x = 0.0;
     intent.force_rotate = 0.0;

  
  switch (GP->GetKey()) 
    {
    case 'q':
      add_special(&intent, INTENT_END_GAME);
      break;

    case '1':
      add_special(&intent, INTENT_ACTION1);
      break;

    case '2':
      add_special(&intent, INTENT_ACTION2);
      break;

    case '3':
      add_special(&intent, INTENT_ACTION3);
      break;

    case '4':
      add_special(&intent, INTENT_ACTION4);
      break;

    case '5':
      add_special(&intent, INTENT_ACTION5);
      break;

    default:
      break;
    }


     mouse_button = GP->GetMouse(&mouse_x, &mouse_y);

     if (mouse_button & 1)
       add_special(&intent, INTENT_JUMP);
     if(mouse_button & 2)
       running=True;
     if (mouse_button & 4)
       strafing=True;
     else
	  strafing=False;  

     if (strafing)
	  intent.force_y += (double) (2 * (MOUSE_XRANGE / 2 - mouse_x)) /
	       (double) MOUSE_XRANGE;
     else
	  intent.force_rotate += (double) (2 * (MOUSE_XRANGE / 2 - mouse_x)) /
	       (double) MOUSE_XRANGE;
     intent.force_x += (double) (2 * (MOUSE_YRANGE / 2 - mouse_y)) /
	  (double) MOUSE_YRANGE;
     
     intent.force_x *= fabs(intent.force_x);
     intent.force_y *= fabs(intent.force_y);
     intent.force_rotate *= intent.force_rotate * intent.force_rotate;;

     if (fabs(intent.force_x) < 0.1)
	  intent.force_x = 0.0;
     if (fabs(intent.force_y) < 0.1)
	  intent.force_y = 0.0;
     if (fabs(intent.force_rotate) < 0.1)
	  intent.force_rotate = 0.0;

     intent.force_x *= 16.0;
     intent.force_y *= 16.0;
     intent.force_rotate *= 10.0;

     if (running) {
          intent.force_x *= 2.0;
          intent.force_y *= 2.0;
          intent.force_z *= 2.0;
          intent.force_rotate *= 2.0;
     }

     return &intent;
}


void add_special(Intent *intent, int special)
{
     if (intent->n_special < MAX_SPECIAL_INTENTIONS) {
	  intent->special[intent->n_special] = special;
	  intent->n_special++;
     }
}
