/*
 *
 * gcomp.h
 *
 */

#ifndef GCOMP_H
#define GCOMP_H

#include "flags.h"
#include "rect.h"
#include "gdefs.h"

const int ATTRIB_STACK_SZ = 100;

const int MSG_NONE      = 0;
const int MSG_QUIET     = 0;
const int MSG_ERROR     = 1;
const int MSG_WARNING   = 2;
const int MSG_INFO      = 3;
const int MSG_RESOURCE  = 4;
const int MSG_FUNCTION  = 4;
const int MSG_ALL       = 100;

class GraphicComponent
{
  int _verbose_level;
  int _verbose_request;
  
  Flags _attrib_stack[100];
  int   _attrib_stack_idx;

 protected:

  Flags _last_attributes;
  char *_proc_name;

  static ScrCoord   _width, _height;
  static ColorIndex *_graph_mem;
  static ZCoord     *_z_mem;
  static ColorIndex _colormap[GP_COLORMAP_SZ];
  static PixelValue _truecolormap[GP_COLORMAP_SZ];
  static int        _bytes_per_line;
  static int        _bits_per_pixel;
  static int        _bytes_per_pixel;
  static int        _R_mask;
  static int        _G_mask;
  static int        _B_mask;
  static Rect       _clip;

 public:

  GraphicComponent(void);

  void Verbose(const int verbose_level);

  GraphicComponent& operator [] (const int verbose_request);
  GraphicComponent& operator << (const char *str);
  GraphicComponent& operator << (const char c);
  GraphicComponent& operator << (const int  i);
  GraphicComponent& operator << (const long i);
  GraphicComponent& operator << (const float  f);
  GraphicComponent& operator << (const double f);

  void PushAttributes(void);
  void PopAttributes(void);
  Flags&  Attributes(void);
  
  virtual void CommitAttributes(void) = 0;
  virtual ~GraphicComponent(void);

};


#include "gcomp_i.h"

#endif







