/*
 *
 * gcomp_i.h
 *
 */

#ifndef GCOMP_I_H
#define GCOMP_I_H

#include <iostream.h>
#include <stdlib.h>

inline GraphicComponent::GraphicComponent()
{
  _verbose_level    = 0;
  _verbose_request  = 0;
  _attrib_stack_idx = 0;
  _last_attributes  = 0;
  _proc_name        = "Unknown component";
}

inline void GraphicComponent::Verbose(const int verbose_level)
{
  _verbose_level = verbose_level;
}

inline GraphicComponent& GraphicComponent::operator[](const int verbose_request)
{
  _verbose_request = verbose_request;

  if(_verbose_level >= _verbose_request)
    cerr << "[TAGL] " << _proc_name << ":: ";
  return *this;
}

inline GraphicComponent& GraphicComponent::operator<<(const char *str) 
{
  if(_verbose_level >= _verbose_request)
    cerr << str;
  return *this;
}

inline GraphicComponent& GraphicComponent::operator<<(const char c) 
{
  if(_verbose_level >= _verbose_request)
    cerr << c;
  return *this;
}


inline GraphicComponent& GraphicComponent::operator<<(const int i) 
{
  if(_verbose_level >= _verbose_request)
    cerr << i;
  return *this;
}


inline GraphicComponent& GraphicComponent::operator<<(const long i) 
{
  if(_verbose_level >= _verbose_request)
    cerr << i;
  return *this;
}


inline GraphicComponent& GraphicComponent::operator<<(const float f)
{
  if(_verbose_level >= _verbose_request)
    cerr << f;
  return *this;
}

inline GraphicComponent& GraphicComponent::operator<<(const double f)
{
  if(_verbose_level >= _verbose_request)
    cerr << f;
  return *this;
}

inline void GraphicComponent::PushAttributes(void)
{
  if(_attrib_stack_idx >= ATTRIB_STACK_SZ - 1)
    {
      (*this)[MSG_ERROR] << "attribute stack overflow error\n";
      exit(0);
    }

  _attrib_stack_idx++;
  _attrib_stack[_attrib_stack_idx] = _attrib_stack[_attrib_stack_idx - 1];
}

inline void GraphicComponent::PopAttributes(void)
{
  _last_attributes = Attributes();
  if(!_attrib_stack_idx)
    {
      (*this)[MSG_ERROR] << "attribute stack underflow error\n";
      exit(0);
    }
  _attrib_stack_idx--;
  CommitAttributes();
}

inline Flags& GraphicComponent::Attributes(void)
{
  return _attrib_stack[_attrib_stack_idx];
}

#endif
