/*
 *
 * gmath.h
 *
 */

#ifndef GMATH_H
#define GMATH_H

#include <iostream.h>

typedef double dbl;

class Coeff
{
  dbl v;

 public:

  Coeff(void);
  Coeff(const double x); 
  Coeff(const long   x);

  operator double() const;
  operator long() const;

  Coeff& operator += (Coeff& X);
  Coeff& operator -= (Coeff& X);
  Coeff& operator *= (Coeff& X);
  Coeff& operator *= (long   x);
  Coeff& operator *= (double x);
  Coeff& operator /= (Coeff& X);
  Coeff& operator /= (long   x);
  Coeff& operator /= (double x);


  Coeff& mac(Coeff& X, Coeff& Y);
  Coeff& mac(long x, long y); 
  Coeff& mac(long x, Coeff& Y);
  Coeff& mac(Coeff& X, long y);

  friend ostream& operator << (ostream&, Coeff&);

  friend class Coord;
  friend class Point;
  friend class Vector;
  friend class Matrix;
};


class Coord
{
  dbl v;

 public:

  Coord(void);
  Coord(const double x); 
  Coord(const long   x);

  operator double() const;
  operator long() const;

  Coord& operator += (Coord& X);
  Coord& operator -= (Coord& X);

  Coord& operator *= (Coord& X);
  Coord& operator *= (long   x);
  Coord& operator *= (double x);


  Coord& operator /= (Coord& X);
  Coord& operator /= (long   x);
  Coord& operator /= (double x);


  Coord& mac(Coord& X, Coord& Y); 
  Coord& mac(long x, long y);    
  Coord& mac(long x, Coord& Y);  
  Coord& mac(Coord& X, long y);
  Coord& mac(Coord& X, Coeff& Y);
  Coord& mac(Coeff& X, Coord& Y);
  Coord& mac(Coeff& X, Coeff& Y); 

  friend ostream& operator << (ostream&, Coord&);
  friend class Point;
  friend class Vector;
  friend class Matrix;
};


class Matrix
{
 public:
  Matrix(void);
  Coeff v[16];
  Coeff& item(int i, int j);

  void Clear(void);
  void Identity(void);

  void Translate(Coeff Tx, Coeff Ty, Coeff Tz);
  void Scale(Coeff Sx, Coeff Sy, Coeff Sz);

  friend ostream& operator << (ostream&, Matrix&);
};


/*
 * base class for points and vectors 
 */

class Coord4
{

 protected:
  Coord v[4];

 public:

  Coord4(Coord x, Coord y, Coord z, Coord t);
  Coord& operator[](int i);

  friend class Matrix;
  friend ostream& operator << (ostream&, Coord4&);
};



class Vector : public Coord4
{
 public:
  Vector(Coord x, Coord y, Coord z);      
};


class Point : public Coord4
{
 public:
  Point(Coord x, Coord y, Coord z);
  Point(Coord x, Coord y, Coord z, Coord t);
};



#include "gmath_i.h"

#endif

