/*
 *
 * gmath_i.h
 *
 */

#ifndef GMATH_I_H
#define GMATH_I_H

////
////
//
// Class Coeff
//
////
////

inline Coeff::Coeff(const double x) {v = x;         } 
inline Coeff::Coeff(const long   d) {v = (dbl)d;    }
inline Coeff::Coeff(void)           {}

inline Coeff::operator double() const { return v;       }
inline Coeff::operator long()   const { return (long)v; }

inline Coeff& Coeff::operator +=(Coeff& X) {v += X.v; return *this; }
inline Coeff& Coeff::operator -=(Coeff& X) {v -= X.v; return *this; }

inline Coeff& Coeff::operator *=(Coeff& X) {v *= X.v; return *this; }
inline Coeff& Coeff::operator *=(long   d) {v *= (dbl)d; return *this; }
inline Coeff& Coeff::operator *=(double x) {v *= x; return *this; }

inline Coeff& Coeff::operator /=(Coeff& X) {v /= X.v; return *this; }
inline Coeff& Coeff::operator /=(long   d) {v /= (dbl)d; return *this; }
inline Coeff& Coeff::operator /=(double x) {v /= x; return *this; }

inline Coeff& Coeff::mac(Coeff& X, Coeff& Y) 
{
  v += (X.v * Y.v); 
  return *this; 
}

inline Coeff& Coeff::mac(long x, long y) 
{
  v += ((dbl)x*(dbl)y); 
  return *this;
} 

inline Coeff& Coeff::mac(long x, Coeff& Y)
{
  v += ((dbl)x * Y.v);
  return *this;
}

inline Coeff& Coeff::mac(Coeff& X, long y)
{
  v += (X.v * (dbl)y);
  return *this;
}

inline ostream& operator << (ostream& s, Coeff& X)
{
  return s << (float)X; 
}

////
////
//
// Class Coord
//
////
////

inline Coord::Coord(void)           {}
inline Coord::Coord(const double x) {v = x;      } 
inline Coord::Coord(const long   x) {v = (dbl)x; }

inline Coord::operator double() const { return v;       }
inline Coord::operator long()   const { return (long)v; }

inline Coord& Coord::operator += (Coord& X)  { v += X.v; return *this; }
inline Coord& Coord::operator -= (Coord& X)  { v -= X.v; return *this; }

inline Coord& Coord::operator *= (Coord& X)  { v *= X.v; return *this; }
inline Coord& Coord::operator *= (long   x)  { v *= (dbl)x; return *this; }
inline Coord& Coord::operator *= (double x)  { v *= x; return *this; }

inline Coord& Coord::operator /= (Coord& X)  { v /= X.v; return *this; }
inline Coord& Coord::operator /= (long   x)  { v /= (dbl)x; return *this; }
inline Coord& Coord::operator /= (double x)  { v /= x; return *this; }

inline Coord& Coord::mac(Coord& X, Coord& Y) 
{ 
  v += (X.v * Y.v); 
  return *this; 
}

inline Coord& Coord::mac(long x, long y)     
{ 
  v += ((dbl)x * (dbl)y);
  return *this; 
}

inline Coord& Coord::mac(long x, Coord& Y)   
{ 
  v += ((dbl)x * Y.v);
  return *this; 
}

inline Coord& Coord::mac(Coord& X, long y)   
{ 
  v += (X.v * (dbl)y);
  return *this; 
}


inline Coord& Coord::mac(Coord& X, Coeff& Y) 
{ 
  v += (X.v * Y.v);
  return *this; 
}

inline Coord& Coord::mac(Coeff& X, Coord& Y) 
{ 
  v += (X.v * Y.v);
  return *this; 
}

inline Coord& Coord::mac(Coeff& X, Coeff& Y) 
{ 
  v += (X.v * Y.v);
  return *this; 
}

inline ostream& operator << (ostream& s, Coord& X)
{
  return s << (float)X; 
}

////
////
//
// Matrix class
//
////
////

inline Matrix::Matrix() {}


inline Coeff& Matrix::item(int i, int j)
{
  return v[(i << 2) + j];
}

inline ostream& operator << (ostream& o, Matrix& M)
{
  int i,j;
  for(i=0; i<4; i++)
    {
      o = o << "[ ";
      for(j=0; j<4; j++)
	o = o << M.item(i,j) << " ";
      o = o << "]\n";
    }
  return o;
}

inline void Matrix::Clear(void)
{
  for(int i=0; i<16; i++)
    v[i] = 0;
}

inline void Matrix::Identity(void)
{
  int i,j;
  for(i = 0; i<4; i++)
    for(j = 0; j<4; j++)
      item(i,j) = ((i==j) ? 1 : 0);
}

inline void Matrix::Translate(Coeff Tx, Coeff Ty, Coeff Tz)
{
  for(int j=0; j<4; j++)
    {
      item(3,j).mac(Tx,item(0,j));
      item(3,j).mac(Ty,item(1,j));
      item(3,j).mac(Tz,item(2,j));
    }
}

inline void Matrix::Scale(Coeff Sx, Coeff Sy, Coeff Sz)
{
  for(int j=0; j<4; j++)
    {
      item(0,j) *= Sx;
      item(1,j) *= Sy;
      item(2,j) *= Sz;
    }
}

////
////
//
// Vector and Point classes
//
////
////

inline Coord4::Coord4(Coord x, Coord y, Coord z, Coord t) 
{
  v[0] = x; 
  v[1] = y; 
  v[2] = z; 
  v[3] = t;
}

inline Coord& Coord4::operator[](int i) { return v[i]; }

inline ostream& operator << (ostream& s, Coord4& V)
{
  return s << "[ " << V[0] << " " 
                   << V[1] << " " 
		   << V[2] << " "
                   << V[3] << " ]\n"; 
}


inline Vector::Vector(Coord x, Coord y, Coord z) : Coord4(x,y,z,0)  {}

inline Point::Point(Coord x, Coord y, Coord z) : Coord4(x,y,z,1) {}
inline Point::Point(Coord x, Coord y, Coord z, Coord t) : Coord4(x,y,z,t) {}

#endif
