/*
 *
 * gport.h
 * Base class for GraphicPorts.
 *
 */


#ifndef GPORT_H
#define GPORT_H

#include <stdlib.h>
#include "gdefs.h"
#include "flags.h"
#include "colorcell.h" 
#include "gcomp.h"

const Flag GPE_OK    = 0;

const Flag GPA_NONE  = 0;
const Flag GPA_DBUFF = 1;
const Flag GPA_ZBUFF = 2;
const Flag GPA_CMAP  = 3;
const Flag GPA_RGB   = 4;

const ColorIndex BLACK   = 0;
const ColorIndex RED     = 1;
const ColorIndex GREEN   = 2;
const ColorIndex YELLOW  = 3;
const ColorIndex BLUE    = 4;
const ColorIndex MAGENTA = 5;
const ColorIndex CYAN    = 6;
const ColorIndex WHITE   = 7;

class GraphicPort: public GraphicComponent
{
  
 protected:


  char       *_name;
  ColorCell   _colortable[GP_COLORMAP_SZ];
  int         _error_code;

 public:

  GraphicPort(char *name, ScrCoord width, ScrCoord height, int verbose_level = 0);

  virtual ~GraphicPort(void) = 0;


  virtual void RGBMode(void)      = 0;
  virtual void ColormapMode(void) = 0;

  virtual int SingleBuffer(void)     = 0;
  virtual int DoubleBuffer(void)     = 0;
  virtual int SwapBuffers(void)      = 0;
  virtual int ZBuffer(const int yes) = 0;

  ColorIndex* GraphMem(void)     const;
  ColorIndex* Colormap(void)     const;
  ZCoord*     ZMem(void)         const;    
  ScrCoord    Width(void)        const;
  ScrCoord    Height(void)       const;
  int         BytesPerLine(void) const;
  Rect&       Clip(void);
  int         BitsPerPixel(void)  const;
  int         BytesPerPixel(void) const;

  Flag        ErrorCode(void)    const;

  void GetGeometry(ScrCoord *_width, ScrCoord *_height) const;

  virtual void MapColor(const ColorIndex i, 
			const ColorComponent r, 
			const ColorComponent g, 
			const ColorComponent b ) = 0;

  virtual int  SetGeometry(const ScrCoord _width, const ScrCoord _height) = 0;

  virtual int  WaitEvent(void) = 0;
  virtual char GetKey(void) = 0;
  virtual int  GetMouse(ScrCoord *x, ScrCoord *y) = 0;

};



// virtual constructor class

const int GP_VC_NORMAL = 0;
const int GP_VC_HANGS  = 1;

typedef GraphicPort *(*GraphicPort_MakeFun)(char*, ScrCoord, ScrCoord, int);

typedef struct
{
  GraphicPort_MakeFun make;
  int flags;
} GraphicPort_VC_Entry;

class GraphicPort_VCC
{
  static int _entries;
  GraphicPort_VC_Entry _entry[VC_ENTRIES];
 public:
  GraphicPort_VCC(void);
  void Register(const GraphicPort_MakeFun make, const int flags);
  GraphicPort* Make(char*, ScrCoord, ScrCoord, int);
};

extern GraphicPort_VCC GraphicPort_VC;

GraphicPort* MakeGraphicPort(char* name, ScrCoord width, ScrCoord height, int = 0);

#include "gport_i.h"

#endif
