/*
 *
 * gproc_i.h
 *
 */

#ifndef GPROC_I_H
#define GPROC_I_H


inline GraphicProcessor::GraphicProcessor(GraphicPort *gp)
{
  _gport = gp;
  _proc_name = "Unknown processor";
}

inline ScrCoord GraphicProcessor::Width(void) const
{
  return _width;
}


inline ScrCoord GraphicProcessor::Height(void) const
{
  return _height;
}

inline  ColorIndex* GraphicProcessor::GraphMem(void)     const
{
  return _graph_mem;
}

inline  ColorIndex* GraphicProcessor::Colormap(void)     const
{
  return _colormap;
}

inline  ZCoord*     GraphicProcessor::ZMem(void)         const
{
  return _z_mem;
}

inline  int         GraphicProcessor::BytesPerLine(void) const
{
  return _bytes_per_line;
}


inline  int         GraphicProcessor::BitsPerPixel(void) const
{
  return _bits_per_pixel;
}

inline  int         GraphicProcessor::RedMask(void)      const
{
  return _R_mask;
}

inline  int         GraphicProcessor::GreenMask(void)    const
{
  return _G_mask;
}

inline  int         GraphicProcessor::BlueMask(void)     const
{
  return _B_mask;
}

inline  Rect&       GraphicProcessor::Clip(void)
{
  return _clip;
}

inline void GraphicProcessor::ColormapMode(void)
{
  _last_attributes = Attributes();
  Attributes().Reset(GA_RGB);
  CommitAttributes();
}

inline void GraphicProcessor::RGBMode(void)
{
  _last_attributes = Attributes();
  Attributes().Set(GA_RGB);
  CommitAttributes();
}

inline void GraphicProcessor::Gouraud(int yes)
{
  _last_attributes = Attributes();
  if(yes)
    Attributes().Set(GA_GOURAUD);
  else
    Attributes().Reset(GA_GOURAUD);
  CommitAttributes();
}

inline void GraphicProcessor::Flat(void)
{
  _last_attributes = Attributes();
  Attributes().Reset(GA_RGB);
  CommitAttributes();
}

inline void GraphicProcessor::Dither(int yes)
{
  _last_attributes = Attributes();
  if(yes)
    Attributes().Set(GA_DITHER);
  else
    Attributes().Reset(GA_DITHER);
  CommitAttributes();
}

inline void GraphicProcessor::ZBuffer(int yes)
{
  _last_attributes = Attributes();
  if(yes)
    Attributes().Set(GA_ZBUFFER);
  else
    Attributes().Reset(GA_ZBUFFER);
  CommitAttributes();
}

inline void GraphicProcessor::Alpha(int yes)
{
  _last_attributes = Attributes();
  if(yes)
    Attributes().Set(GA_ALPHA);
  else
    Attributes().Reset(GA_ALPHA);
  CommitAttributes();
}

inline void GraphicProcessor::Texture(int yes)
{
  _last_attributes = Attributes();
  if(yes)
    Attributes().Set(GA_TEXTURE);
  else
    Attributes().Reset(GA_TEXTURE);
  CommitAttributes();
}



#endif
