/*
 *
 * polyeng.h
 *
 */


#ifndef POLYENG_H
#define POLYENG_H

#include "gdefs.h"
#include "vpool.h"
#include "polygon.h"
#include "gproc.h"

const int PE_MAX_HEIGHT = 1024;

const int D_SHIFT  = 4;
const int D_MASK   = 15;
const int D_PMASK  = 3;
const int D_PSHIFT = 2;

const int R_SHIFT  = 4;
const int G_SHIFT  = 2;
const int B_SHIFT  = 0;

const int PEA_ZCLIP = GA_MAX + 1;

typedef struct 
{
  Vertex _left;
  Vertex _right;
  int    _init;
} Mug;


class PolygonEngine;

typedef void (* PolyFun)(VertexAttributes *);
typedef void (* LineFun)(VertexAttributes *, Vertex *, Vertex *);
typedef void (* PixelFun)(VertexAttributes *, ScrCoord, ScrCoord);

class PolygonEngine : public GraphicProcessor
{

 protected:
  VertexAttributes    _va;

  static VertexPool _vpool;
  static Polygon    _p1;
  static Polygon    _p2;
  static Mug        _mug[PE_MAX_HEIGHT];

  static   int D4[16]; // Dithering matrix

  VertexAttributes _vattrib_stack[ATTRIB_STACK_SZ];

  PolyFun   _fillpoly;
  LineFun   _drawline;
  PixelFun  _setpixel;

 public:
  PolygonEngine(GraphicPort *gp, int verbose_level = MSG_NONE);
  virtual ~PolygonEngine(void);
  virtual void CommitAttributes(void) = 0;
  VertexAttributes& VAttributes(void);

  void Push(Vertex *v);
  void Reset(void);

  void FillPoly();
  void DrawPoly();

  void SetPixel(ScrCoord x, ScrCoord y);
  void SetPixel(ScrCoord x, ScrCoord y, ColorIndex c);
  void SetPixel(ScrCoord x, ScrCoord y, 
		ColorComponent r, ColorComponent g, ColorComponent b);

  void ClipPoly(void);

};


// virtual constructor class

typedef PolygonEngine *(*PolygonEngine_MakeFun)(GraphicPort*, int);

class PolygonEngine_VCC
{
  static int _entries;
  PolygonEngine_MakeFun _entry[VC_ENTRIES];
 public:
  PolygonEngine_VCC(void);
  void Register(const PolygonEngine_MakeFun make);
  PolygonEngine* Make(GraphicPort *GP, int verb);
};

extern PolygonEngine_VCC PolygonEngine_VC;

PolygonEngine* MakePolygonEngine(GraphicPort *GP, int verb = 0);

#include "polyeng_i.h"

#endif
