/*
 *
 * Xgport.h
 * A GraphicPort for X11
 *
 */


#ifndef XGPORT_H
#define XGPORT_H

#include "gport.h"
#include <iostream.h>

const int D_WIDTH    = 640;
const int D_HEIGHT   = 480;
const int MIN_WIDTH  = 100;
const int MIN_HEIGHT = 100;

const Flag XGPR_NONE            = 0;
const Flag XGPR_DISPLAY         = 1;
const Flag XGPR_SCREEN          = 2;
const Flag XGPR_WINDOW          = 3;
const Flag XGPR_GLOBAL_CMAP     = 4;
const Flag XGPR_LOCAL_CMAP      = 5;
const Flag XGPR_SHM_XIMAGE      = 6;
const Flag XGPR_X_ERROR_HANDLER = 7;
const Flag XGPR_STD_XIMAGE      = 8;
const Flag XGPR_ZBUFFER         = 9;

const Flag XGPE_DISPLAY = 1;
const Flag XGPE_VISUAL  = 2;
const Flag XGPE_WINDOW  = 3;
const Flag XGPE_XIMAGE  = 4;
const Flag XGPE_MALLOC  = 5;

const ScrCoord MAX_X_WIDTH  = 640;
const ScrCoord MAX_X_HEIGHT = 480;

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#ifdef X_SHM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>

// I could not find the following prototypes in system 
// includes (I work with Linux),
// These declarations avoid a compiler warning.

extern "C" {
  int XShmQueryExtension(Display *);
  int XShmGetEventBase(Display *);
  }
#endif


#include <stdlib.h>
#include <stdio.h>

class XGraphicPort : public GraphicPort
{
 public:

  XGraphicPort(char *name, ScrCoord width, ScrCoord height, int verbose_level = 0);
  virtual ~XGraphicPort(void);
  virtual void CommitAttributes(void);


  virtual void RGBMode(void);
  virtual void ColormapMode(void);

  virtual int SingleBuffer(void);
  virtual int DoubleBuffer(void);
  virtual int SwapBuffers(void);
  virtual int ZBuffer(const int yes);

  virtual void MapColor(const ColorIndex i, 
			const ColorComponent r, 
			const ColorComponent g, 
			const ColorComponent b );  

  virtual int SetGeometry(const ScrCoord width, const ScrCoord height);

  virtual char GetKey(void);
  virtual int  GetMouse(ScrCoord *x, ScrCoord *y);
  virtual int  WaitEvent(void);

 protected: 

                // Predicate procedure used by WaitEvent()
  static Bool predproc(Display *display, XEvent *event, char *arg);

                // X error handler
  static int HandleXError(Display *, XErrorEvent *);
  void InstallXErrorHandler(void);
  void UninstallXErrorHandler(void);

                // internal functions
  int  OpenWindow(char *display_name = "");

  int  AllocateColormaps(void);
  void ResetColormaps(void);
  void ResetColortable(void);
  void LocalColormap(void);

  int  AllocateXImage(void);
  void FreeXImage(void);

  int  AllocateZBuffer(void);
  void FreeZBuffer(void);

  int  Resize(const ScrCoord width, const ScrCoord height);



#ifdef X_SHM
  XShmSegmentInfo _shminfo;
  int _completion_type;
#endif

#include <X11/Xlib.h>
#include <X11/Xutil.h>

  XImage  *_ximage;       
  Display *_display;
  int      _screen;
  Window   _window;
  GC       _gc;
//  Colormap _global_cmap;
//  Colormap _local_cmap;
  int      _local_cmap_flag;
  int      _shm_flag;
  Flags    _resources;
  static   int _xerror_flag;
  int      _buffered;
  char     _charbuff;

};


#include "Xgport_i.h"

#endif
