
/*
 *
 * polyeng.C
 *
 */


#include "polyeng_24.h"
#include <string.h>

typedef struct
{
  unsigned char b;
  unsigned char g;
  unsigned char r;
} Pixel24;


////
////
//
// Virtual constructor stuff
//
////
////


PolygonEngine *MakePolygonEngine_24(GraphicPort *GP, int verb)
{
  PolygonEngine *PE = new PolygonEngine_24(GP,verb);

  if(PE->BitsPerPixel() != 24 ||
     PE->RedMask()      != (255 << 16) ||
     PE->GreenMask()    != (255 << 8)  ||
     PE->BlueMask()     != 255          )
    {
      delete PE;
      return NULL;
    }
  return PE;
}

class PolygonEngine_24Stub
{
public:
  PolygonEngine_24Stub(void) 
    { 
      PolygonEngine_VC.Register(MakePolygonEngine_24); 
    } 
};

static PolygonEngine_24Stub Dummy;


PolyFun PolygonEngine_24::_fillpoly_array[16] =
{
//  TEXTURE ALPHA ZBUFFER DITHER GOURAUD RGB
//
//     0      0      0      0      0      0
  PolygonEngine_24::FillPoly_c,
//     0      0      0      0      0      1
  PolygonEngine_24::FillPoly_rgb,
//     0      0      0      0      1      0
  PolygonEngine_24::FillPoly_C,
//     0      0      0      0      1      1
  PolygonEngine_24::FillPoly_RGB,
//     0      0      0      1      0      0
  PolygonEngine_24::FillPoly_c_D,
//     0      0      0      1      0      1
  PolygonEngine_24::FillPoly_rgb,
//     0      0      0      1      1      0
  PolygonEngine_24::FillPoly_C_D,
//     0      0      0      1      1      1
  PolygonEngine_24::FillPoly_RGB,

//  TEXTURE ALPHA ZBUFFER DITHER GOURAUD RGB
//
//     0      0      1      0      0      0
  PolygonEngine_24::FillPoly_c_Z,
//     0      0      1      0      0      1
  PolygonEngine_24::FillPoly_rgb_Z,
//     0      0      1      0      1      0
  PolygonEngine_24::FillPoly_C_Z,
//     0      0      1      0      1      1
  PolygonEngine_24::FillPoly_RGB_Z,
//     0      0      1      1      0      0
  PolygonEngine_24::FillPoly_c_Z_D,
//     0      0      1      1      0      1
  PolygonEngine_24::FillPoly_rgb_Z,
//     0      0      1      1      1      0
  PolygonEngine_24::FillPoly_C_Z_D,
//     0      0      1      1      1      1
  PolygonEngine_24::FillPoly_RGB_Z
};


LineFun PolygonEngine_24::_drawline_array[16] =
{
//  TEXTURE ALPHA ZBUFFER DITHER GOURAUD RGB
//
//     0      0      0      0      0      0
  PolygonEngine_24::DrawLine_c,
//     0      0      0      0      0      1
  PolygonEngine_24::DrawLine_rgb,
//     0      0      0      0      1      0
  PolygonEngine_24::DrawLine_C,
//     0      0      0      0      1      1
  PolygonEngine_24::DrawLine_RGB,
//     0      0      0      1      0      0
  PolygonEngine_24::DrawLine_c_D,
//     0      0      0      1      0      1
  PolygonEngine_24::DrawLine_rgb,
//     0      0      0      1      1      0
  PolygonEngine_24::DrawLine_C_D,
//     0      0      0      1      1      1
  PolygonEngine_24::DrawLine_RGB,

//  TEXTURE ALPHA ZBUFFER DITHER GOURAUD RGB
//
//     0      0      1      0      0      0
  PolygonEngine_24::DrawLine_c_Z,
//     0      0      1      0      0      1
  PolygonEngine_24::DrawLine_rgb_Z,
//     0      0      1      0      1      0
  PolygonEngine_24::DrawLine_C_Z,
//     0      0      1      0      1      1
  PolygonEngine_24::DrawLine_RGB_Z,
//     0      0      1      1      0      0
  PolygonEngine_24::DrawLine_c_Z_D,
//     0      0      1      1      0      1
  PolygonEngine_24::DrawLine_rgb_Z,
//     0      0      1      1      1      0
  PolygonEngine_24::DrawLine_C_Z_D,
//     0      0      1      1      1      1
  PolygonEngine_24::DrawLine_RGB_Z
};



PixelFun PolygonEngine_24::_setpixel_array[16] =
{
//  TEXTURE ALPHA ZBUFFER DITHER GOURAUD RGB
//
//     0      0      0      0      0      0
  PolygonEngine_24::SetPixel_c,
//     0      0      0      0      0      1
  PolygonEngine_24::SetPixel_rgb,
//     0      0      0      0      1      0
  PolygonEngine_24::SetPixel_c,
//     0      0      0      0      1      1
  PolygonEngine_24::SetPixel_rgb,
//     0      0      0      1      0      0
  PolygonEngine_24::SetPixel_c_D,
//     0      0      0      1      0      1
  PolygonEngine_24::SetPixel_rgb,
//     0      0      0      1      1      0
  PolygonEngine_24::SetPixel_c_D,
//     0      0      0      1      1      1
  PolygonEngine_24::SetPixel_rgb,

//  TEXTURE ALPHA ZBUFFER DITHER GOURAUD RGB
//
//     0      0      1      0      0      0
  PolygonEngine_24::SetPixel_c_z,
//     0      0      1      0      0      1
  PolygonEngine_24::SetPixel_rgb_z,
//     0      0      1      0      1      0
  PolygonEngine_24::SetPixel_c_z,
//     0      0      1      0      1      1
  PolygonEngine_24::SetPixel_rgb_z,
//     0      0      1      1      0      0
  PolygonEngine_24::SetPixel_c_z_D,
//     0      0      1      1      0      1
  PolygonEngine_24::SetPixel_rgb_z,
//     0      0      1      1      1      0
  PolygonEngine_24::SetPixel_c_z,
//     0      0      1      1      1      1
  PolygonEngine_24::SetPixel_rgb_z
};


PolygonEngine_24::PolygonEngine_24(GraphicPort *gp, int verb) : PolygonEngine(gp, verb)
{
  _proc_name = "PolygonEngine_24";
  Verbose(verb);
  (*this)[MSG_INFO] << "hello, world!\n";
  CommitAttributes();
}

PolygonEngine_24::~PolygonEngine_24(void)
{
}

void PolygonEngine_24::CommitAttributes(void)
{
  if(Attributes().GetAll() > 15)
    {
      (*this)[MSG_ERROR] << "invalid attributes\n";
      return;
    }

  _fillpoly = _fillpoly_array[Attributes().GetAll()];
  _drawline = _drawline_array[Attributes().GetAll()];
  _setpixel = _setpixel_array[Attributes().GetAll()];

}  




#define GENFILL_RGB_MAX    255
#define GENFILL_RGB_SHIFT  0
#define GENFILL_NAME       PolygonEngine_24::FillPoly_rgb
#define GENFILL_PIXEL      Pixel24
#define GENFILL_HEAD       ColorComponent r = MIN(VA->r,GENFILL_RGB_MAX) >> GENFILL_RGB_SHIFT; \
                           ColorComponent g = MIN(VA->g,GENFILL_RGB_MAX) >> GENFILL_RGB_SHIFT; \
                           ColorComponent b = MIN(VA->b,GENFILL_RGB_MAX) >> GENFILL_RGB_SHIFT;
#define GENFILL_DO_PIXEL   graph_ptr->b = b; \
			   graph_ptr->g = g; \
			   graph_ptr->r = r;

#include "genfill.h"




#define GENFILL_Z
#define GENFILL_RGB_MAX    255
#define GENFILL_RGB_SHIFT  0
#define GENFILL_NAME       PolygonEngine_24::FillPoly_rgb_Z
#define GENFILL_PIXEL      Pixel24
#define GENFILL_HEAD       ColorComponent r = MIN(VA->r,GENFILL_RGB_MAX) >> GENFILL_RGB_SHIFT; \
                           ColorComponent g = MIN(VA->g,GENFILL_RGB_MAX) >> GENFILL_RGB_SHIFT; \
                           ColorComponent b = MIN(VA->b,GENFILL_RGB_MAX) >> GENFILL_RGB_SHIFT;
#define GENFILL_DO_PIXEL   if(z < *z_ptr)                                  \
                               {                                           \
                                 *z_ptr = z;                               \
				 graph_ptr->b = b;                         \
				 graph_ptr->g = g;                         \
				 graph_ptr->r = r;                         \
			       }

#include "genfill.h"


#define GENFILL_RGB
#define GENFILL_RGB_MAX    255
#define GENFILL_RGB_SHIFT  0
#define GENFILL_NAME       PolygonEngine_24::FillPoly_RGB
#define GENFILL_PIXEL      Pixel24
#define GENFILL_DO_PIXEL   graph_ptr->b = b; \
			   graph_ptr->g = g; \
			   graph_ptr->r = r;
#include "genfill.h"


#define GENFILL_Z
#define GENFILL_RGB
#define GENFILL_RGB_MAX    255
#define GENFILL_RGB_SHIFT  0
#define GENFILL_NAME       PolygonEngine_24::FillPoly_RGB_Z
#define GENFILL_PIXEL      Pixel24
#define GENFILL_DO_PIXEL   if(z < *z_ptr)                                  \
                               {                                           \
                                 *z_ptr = z;                               \
                                 graph_ptr->b = b;                         \
			         graph_ptr->g = g;                         \
			         graph_ptr->r = r;                         \
			       }
#include "genfill.h"



#define GENFILL_C
#define GENFILL_C_MAX      (255 << D_SHIFT)
#define GENFILL_C_SHIFT    D_SHIFT
#define GENFILL_NAME       PolygonEngine_24::FillPoly_C
#define GENFILL_PIXEL      Pixel24
#define GENFILL_DO_PIXEL   graph_ptr->b = _truecolormap[c] & 255;                  \
                           graph_ptr->g = (_truecolormap[c] & (255 << 8 )) >>  8;  \
                           graph_ptr->r = (_truecolormap[c] & (255 << 16)) >> 16;
#include "genfill.h"



#define GENFILL_Z
#define GENFILL_C
#define GENFILL_C_MAX      (255 << D_SHIFT)
#define GENFILL_C_SHIFT    D_SHIFT
#define GENFILL_NAME       PolygonEngine_24::FillPoly_C_Z
#define GENFILL_PIXEL      Pixel24
#define GENFILL_DO_PIXEL   if(z < *z_ptr)                                                \
                               {                                                         \
                                 *z_ptr = z;                                             \
				 graph_ptr->b =  _truecolormap[c] & 255;                 \
                                 graph_ptr->g = (_truecolormap[c] & (255 << 8 )) >>  8;  \
                                 graph_ptr->r = (_truecolormap[c] & (255 << 16)) >> 16;  \
			       }

#include "genfill.h"


#define GENFILL_NAME   PolygonEngine_24::FillPoly_c
#define GENFILL_PIXEL  Pixel24
#define GENFILL_HEAD   ColorComponent r =  _truecolormap[VA->c >> D_SHIFT] & 255;             \
                       ColorComponent g = (_truecolormap[VA->c >> D_SHIFT] & (255<<8 )) >> 8; \
                       ColorComponent b = (_truecolormap[VA->c >> D_SHIFT] & (255<<16)) >> 16;
#define GENFILL_SCAN   for(x=x1; x<=x2; x++,graph_ptr++)  \
                           {                              \
			       graph_ptr->b = b;          \
			       graph_ptr->g = g;          \
			       graph_ptr->r = r;          \
                           }

#include "genfill.h"


#define GENFILL_Z
#define GENFILL_NAME   PolygonEngine_24::FillPoly_c_Z
#define GENFILL_PIXEL  Pixel24
#define GENFILL_HEAD   ColorComponent r =  _truecolormap[VA->c >> D_SHIFT] & 255;             \
                       ColorComponent g = (_truecolormap[VA->c >> D_SHIFT] & (255<<8 )) >> 8; \
                       ColorComponent b = (_truecolormap[VA->c >> D_SHIFT] & (255<<16)) >> 16;
#define GENFILL_DO_PIXEL   if(z < *z_ptr)                                  \
                               {                                           \
                                 *z_ptr = z;                               \
			         graph_ptr->b = b;                         \
			         graph_ptr->g = g;                         \
			         graph_ptr->r = r;                         \
			       }
#include "genfill.h"


#define GENFILL_C
#define GENFILL_C_MAX    (255 << D_SHIFT)
#define GENFILL_C_SHIFT  0
#define GENFILL_NAME     PolygonEngine_24::FillPoly_C_D
#define GENFILL_PIXEL    Pixel24
#define GENFILL_HEAD     ColorCode cw;
#define GENFILL_DO_PIXEL cw = c >> D_SHIFT;                                                  \
                         if((c & D_MASK) > D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)] ) \
                            cw++;                                                            \
                           graph_ptr->b =  _truecolormap[cw] & 255;                          \
                           graph_ptr->g = (_truecolormap[cw] & (255 << 8 )) >>  8;           \
                           graph_ptr->r = (_truecolormap[cw] & (255 << 16)) >> 16;

#include "genfill.h"			 


#define GENFILL_NAME     PolygonEngine_24::FillPoly_c_D
#define GENFILL_PIXEL    Pixel24
#define GENFILL_HEAD     ColorCode c = VA->c;                                                \
                         c = (c > (255 << D_SHIFT)) ? (255 << D_SHIFT) : c;                  \
			 ColorCode cw = c >> D_SHIFT;                                        \
                         ColorCode cm = c & D_MASK;

#define GENFILL_DO_PIXEL if(cm > D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)] )           \
                            {                                                                \
                              graph_ptr->b =  _truecolormap[cw+1] & 255;                     \
                              graph_ptr->g = (_truecolormap[cw+1] & (255 << 8 )) >>  8;      \
                              graph_ptr->r = (_truecolormap[cw+1] & (255 << 16)) >> 16;      \
			    }                                                                \
                         else                                                                \
                            {                                                                \
                              graph_ptr->b =  _truecolormap[cw] & 255;                       \
                              graph_ptr->g = (_truecolormap[cw] & (255 << 8 )) >>  8;        \
                              graph_ptr->r = (_truecolormap[cw] & (255 << 16)) >> 16;        \
			    }       
#include "genfill.h"



#define GENFILL_Z
#define GENFILL_C
#define GENFILL_C_MAX    (255 << D_SHIFT)
#define GENFILL_C_SHIFT  0
#define GENFILL_NAME     PolygonEngine_24::FillPoly_C_Z_D
#define GENFILL_PIXEL    Pixel24
#define GENFILL_HEAD     ColorCode cw;
#define GENFILL_DO_PIXEL if(z < *z_ptr)                  \
                            {                            \
			      *z_ptr = z;                \
			      cw = c >> D_SHIFT;         \
                           if((c & D_MASK) > D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)] ) \
                              cw++;                                                            \
                              graph_ptr->b =  _truecolormap[cw] & 255;                         \
                              graph_ptr->g = (_truecolormap[cw] & (255 << 8 )) >>  8;          \
                              graph_ptr->r = (_truecolormap[cw] & (255 << 16)) >> 16;          \
			    }
#include "genfill.h"



#define GENFILL_Z
#define GENFILL_NAME     PolygonEngine_24::FillPoly_c_Z_D
#define GENFILL_PIXEL    Pixel24
#define GENFILL_HEAD     ColorCode c = VA->c;                                                \
                         c = (c > (255 << D_SHIFT)) ? (255 << D_SHIFT) : c;                  \
			 ColorCode cw = c >> D_SHIFT;                                        \
                         ColorCode cm = c & D_MASK;

#define GENFILL_DO_PIXEL if(z < *z_ptr)                                                      \
                         {                                                                   \
			   *z_ptr = z;                                                       \
                         if(cm > D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)] )           \
                            {                                                                \
                              graph_ptr->b =  _truecolormap[cw+1] & 255;                     \
                              graph_ptr->g = (_truecolormap[cw+1] & (255 << 8 )) >>  8;      \
                              graph_ptr->r = (_truecolormap[cw+1] & (255 << 16)) >> 16;      \
			    }                                                                \
                         else                                                                \
                            {                                                                \
                              graph_ptr->b =  _truecolormap[cw] & 255;                       \
                              graph_ptr->g = (_truecolormap[cw] & (255 << 8 )) >>  8;        \
                              graph_ptr->r = (_truecolormap[cw] & (255 << 16)) >> 16;        \
			    }                                                                \
		         }
#include "genfill.h"



//
//
// DrawLine functions
//
//




#define GENDRAW_RGB_MAX    255
#define GENDRAW_RGB_SHIFT  0
#define GENDRAW_NAME       PolygonEngine_24::DrawLine_rgb
#define GENDRAW_PIXEL      Pixel24
#define GENDRAW_HEAD       ColorComponent r = MIN(VA->r,GENDRAW_RGB_MAX) >> GENDRAW_RGB_SHIFT; \
                           ColorComponent g = MIN(VA->g,GENDRAW_RGB_MAX) >> GENDRAW_RGB_SHIFT; \
                           ColorComponent b = MIN(VA->b,GENDRAW_RGB_MAX) >> GENDRAW_RGB_SHIFT;
#define GENDRAW_DO_PIXEL   graph_ptr->b = b; \
			   graph_ptr->g = g; \
			   graph_ptr->r = r;

#include "gendraw.h"




#define GENDRAW_Z
#define GENDRAW_RGB_MAX    255
#define GENDRAW_RGB_SHIFT  0
#define GENDRAW_NAME       PolygonEngine_24::DrawLine_rgb_Z
#define GENDRAW_PIXEL      Pixel24
#define GENDRAW_HEAD       ColorComponent r = MIN(VA->r,GENDRAW_RGB_MAX) >> GENDRAW_RGB_SHIFT; \
                           ColorComponent g = MIN(VA->g,GENDRAW_RGB_MAX) >> GENDRAW_RGB_SHIFT; \
                           ColorComponent b = MIN(VA->b,GENDRAW_RGB_MAX) >> GENDRAW_RGB_SHIFT;
#define GENDRAW_DO_PIXEL   if(z < *z_ptr)                                  \
                               {                                           \
                                 *z_ptr = z;                               \
				 graph_ptr->b = b;                         \
				 graph_ptr->g = g;                         \
				 graph_ptr->r = r;                         \
			       }

#include "gendraw.h"


#define GENDRAW_RGB
#define GENDRAW_RGB_MAX    255
#define GENDRAW_RGB_SHIFT  0
#define GENDRAW_NAME       PolygonEngine_24::DrawLine_RGB
#define GENDRAW_PIXEL      Pixel24
#define GENDRAW_DO_PIXEL   graph_ptr->b = b; \
			   graph_ptr->g = g; \
			   graph_ptr->r = r;
#include "gendraw.h"


#define GENDRAW_Z
#define GENDRAW_RGB
#define GENDRAW_RGB_MAX    255
#define GENDRAW_RGB_SHIFT  0
#define GENDRAW_NAME       PolygonEngine_24::DrawLine_RGB_Z
#define GENDRAW_PIXEL      Pixel24
#define GENDRAW_DO_PIXEL   if(z < *z_ptr)                                  \
                               {                                           \
                                 *z_ptr = z;                               \
                                 graph_ptr->b = b;                         \
			         graph_ptr->g = g;                         \
			         graph_ptr->r = r;                         \
			       }
#include "gendraw.h"



#define GENDRAW_C
#define GENDRAW_C_MAX      (255 << D_SHIFT)
#define GENDRAW_C_SHIFT    D_SHIFT
#define GENDRAW_NAME       PolygonEngine_24::DrawLine_C
#define GENDRAW_PIXEL      Pixel24
#define GENDRAW_DO_PIXEL   graph_ptr->b = _truecolormap[c] & 255;                  \
                           graph_ptr->g = (_truecolormap[c] & (255 << 8 )) >>  8;  \
                           graph_ptr->r = (_truecolormap[c] & (255 << 16)) >> 16;
#include "gendraw.h"



#define GENDRAW_Z
#define GENDRAW_C
#define GENDRAW_C_MAX      (255 << D_SHIFT)
#define GENDRAW_C_SHIFT    D_SHIFT
#define GENDRAW_NAME       PolygonEngine_24::DrawLine_C_Z
#define GENDRAW_PIXEL      Pixel24
#define GENDRAW_DO_PIXEL   if(z < *z_ptr)                                                \
                               {                                                         \
                                 *z_ptr = z;                                             \
				 graph_ptr->b =  _truecolormap[c] & 255;                 \
                                 graph_ptr->g = (_truecolormap[c] & (255 << 8 )) >>  8;  \
                                 graph_ptr->r = (_truecolormap[c] & (255 << 16)) >> 16;  \
			       }

#include "gendraw.h"


#define GENDRAW_NAME   PolygonEngine_24::DrawLine_c
#define GENDRAW_PIXEL  Pixel24
#define GENDRAW_HEAD   ColorComponent r =  _truecolormap[VA->c >> D_SHIFT] & 255;             \
                       ColorComponent g = (_truecolormap[VA->c >> D_SHIFT] & (255<<8 )) >> 8; \
                       ColorComponent b = (_truecolormap[VA->c >> D_SHIFT] & (255<<16)) >> 16;
#define GENDRAW_DO_PIXEL                            \
                           {                        \
			       graph_ptr->b = b;    \
			       graph_ptr->g = g;    \
			       graph_ptr->r = r;    \
                           }

#include "gendraw.h"


#define GENDRAW_Z
#define GENDRAW_NAME   PolygonEngine_24::DrawLine_c_Z
#define GENDRAW_PIXEL  Pixel24
#define GENDRAW_HEAD   ColorComponent r =  _truecolormap[VA->c >> D_SHIFT] & 255;             \
                       ColorComponent g = (_truecolormap[VA->c >> D_SHIFT] & (255<<8 )) >> 8; \
                       ColorComponent b = (_truecolormap[VA->c >> D_SHIFT] & (255<<16)) >> 16;
#define GENDRAW_DO_PIXEL   if(z < *z_ptr)                                  \
                               {                                           \
                                 *z_ptr = z;                               \
			         graph_ptr->b = b;                         \
			         graph_ptr->g = g;                         \
			         graph_ptr->r = r;                         \
			       }
#include "gendraw.h"


#define GENDRAW_C
#define GENDRAW_C_MAX    (255 << D_SHIFT)
#define GENDRAW_C_SHIFT  0
#define GENDRAW_NAME     PolygonEngine_24::DrawLine_C_D
#define GENDRAW_PIXEL    Pixel24
#define GENDRAW_HEAD     ColorCode cw;
#define GENDRAW_DO_PIXEL cw = c >> D_SHIFT;                                                  \
                         if((c & D_MASK) > D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)] ) \
                            cw++;                                                            \
                           graph_ptr->b =  _truecolormap[cw] & 255;                          \
                           graph_ptr->g = (_truecolormap[cw] & (255 << 8 )) >>  8;           \
                           graph_ptr->r = (_truecolormap[cw] & (255 << 16)) >> 16;

#include "gendraw.h"			 


#define GENDRAW_NAME     PolygonEngine_24::DrawLine_c_D
#define GENDRAW_PIXEL    Pixel24
#define GENDRAW_HEAD     ColorCode c = VA->c;                                                \
                         c = (c > (255 << D_SHIFT)) ? (255 << D_SHIFT) : c;                  \
			 ColorCode cw = c >> D_SHIFT;                                        \
                         ColorCode cm = c & D_MASK;

#define GENDRAW_DO_PIXEL if(cm > D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)] )           \
                            {                                                                \
                              graph_ptr->b =  _truecolormap[cw+1] & 255;                     \
                              graph_ptr->g = (_truecolormap[cw+1] & (255 << 8 )) >>  8;      \
                              graph_ptr->r = (_truecolormap[cw+1] & (255 << 16)) >> 16;      \
			    }                                                                \
                         else                                                                \
                            {                                                                \
                              graph_ptr->b =  _truecolormap[cw] & 255;                       \
                              graph_ptr->g = (_truecolormap[cw] & (255 << 8 )) >>  8;        \
                              graph_ptr->r = (_truecolormap[cw] & (255 << 16)) >> 16;        \
			    }       
#include "gendraw.h"



#define GENDRAW_Z
#define GENDRAW_C
#define GENDRAW_C_MAX    (255 << D_SHIFT)
#define GENDRAW_C_SHIFT  0
#define GENDRAW_NAME     PolygonEngine_24::DrawLine_C_Z_D
#define GENDRAW_PIXEL    Pixel24
#define GENDRAW_HEAD     ColorCode cw;
#define GENDRAW_DO_PIXEL if(z < *z_ptr)                  \
                            {                            \
			      *z_ptr = z;                \
			      cw = c >> D_SHIFT;         \
                           if((c & D_MASK) > D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)] ) \
                              cw++;                                                            \
                              graph_ptr->b =  _truecolormap[cw] & 255;                         \
                              graph_ptr->g = (_truecolormap[cw] & (255 << 8 )) >>  8;          \
                              graph_ptr->r = (_truecolormap[cw] & (255 << 16)) >> 16;          \
			    }
#include "gendraw.h"



#define GENDRAW_Z
#define GENDRAW_NAME     PolygonEngine_24::DrawLine_c_Z_D
#define GENDRAW_PIXEL    Pixel24
#define GENDRAW_HEAD     ColorCode c = VA->c;                                                \
                         c = (c > (255 << D_SHIFT)) ? (255 << D_SHIFT) : c;                  \
			 ColorCode cw = c >> D_SHIFT;                                        \
                         ColorCode cm = c & D_MASK;

#define GENDRAW_DO_PIXEL if(z < *z_ptr)                                                      \
                         {                                                                   \
			   *z_ptr = z;                                                       \
                         if(cm > D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)] )           \
                            {                                                                \
                              graph_ptr->b =  _truecolormap[cw+1] & 255;                     \
                              graph_ptr->g = (_truecolormap[cw+1] & (255 << 8 )) >>  8;      \
                              graph_ptr->r = (_truecolormap[cw+1] & (255 << 16)) >> 16;      \
			    }                                                                \
                         else                                                                \
                            {                                                                \
                              graph_ptr->b =  _truecolormap[cw] & 255;                       \
                              graph_ptr->g = (_truecolormap[cw] & (255 << 8 )) >>  8;        \
                              graph_ptr->r = (_truecolormap[cw] & (255 << 16)) >> 16;        \
			    }                                                                \
		         }
#include "gendraw.h"

//
//
// SetPixel functions
//
//



void PolygonEngine_24::SetPixel_rgb(VertexAttributes *VA, ScrCoord x, ScrCoord y)
{
  Pixel24 *graph_ptr = (Pixel24 *)
    (_graph_mem + y * _bytes_per_line + x * sizeof(Pixel24));

  ColorComponent r = MIN(VA->r,255); 
  ColorComponent g = MIN(VA->g,255); 
  ColorComponent b = MIN(VA->b,255); 

  graph_ptr->r = r;
  graph_ptr->g = g;
  graph_ptr->b = b;
}


void PolygonEngine_24::SetPixel_rgb_z(VertexAttributes *VA, ScrCoord x, ScrCoord y)
{
  ZCoord       *z_ptr     = _z_mem + y * _width + x;

  if(VA->z < *z_ptr)
    {
      *z_ptr = VA->z;

      Pixel24 *graph_ptr = (Pixel24 *)
	(_graph_mem + y * _bytes_per_line + x * sizeof(Pixel24));

      ColorComponent r = MIN(VA->r,255); 
      ColorComponent g = MIN(VA->g,255); 
      ColorComponent b = MIN(VA->b,255); 

      graph_ptr->r = r;
      graph_ptr->g = g;
      graph_ptr->b = b;
    }
}

void PolygonEngine_24::SetPixel_c(VertexAttributes *VA, ScrCoord x, ScrCoord y)
{
  Pixel24 *graph_ptr = (Pixel24 *)
    (_graph_mem + y * _bytes_per_line + x * sizeof(Pixel24));

  *graph_ptr = *(Pixel24 *)&_truecolormap[VA->c >> D_SHIFT];
}

void PolygonEngine_24::SetPixel_c_z(VertexAttributes *VA, ScrCoord x, ScrCoord y)
{
  ZCoord   *z_ptr  = _z_mem + y * _width + x;

  if(VA->z < *z_ptr)
    {
      *z_ptr = VA->z;

      Pixel24 *graph_ptr = (Pixel24 *)
	(_graph_mem + y * _bytes_per_line + x * sizeof(Pixel24));

      *graph_ptr = *(Pixel24 *)&_truecolormap[VA->c >> D_SHIFT];
    }
}

void PolygonEngine_24::SetPixel_c_D(VertexAttributes *VA, ScrCoord x, ScrCoord y)
{
  Pixel24 *graph_ptr = (Pixel24 *)
    (_graph_mem + y * _bytes_per_line + x * sizeof(Pixel24));

  ColorCode  c = MIN(VA->c, 255 << D_SHIFT);
  ColorCode cc = c >> D_SHIFT;

  int treshold = D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)];  

  if((c & D_MASK) > treshold)
    cc++;

  *graph_ptr = *(Pixel24 *)&_truecolormap[cc];
}



void PolygonEngine_24::SetPixel_c_z_D(VertexAttributes *VA, ScrCoord x, ScrCoord y)
{

  ZCoord   *z_ptr  = _z_mem + y * _width + x;

  if(VA->z < *z_ptr)
    {
      *z_ptr = VA->z;

      Pixel24 *graph_ptr = (Pixel24 *)
	(_graph_mem + y * _bytes_per_line + x * sizeof(Pixel24));

      ColorCode  c = MIN(VA->c, 255 << D_SHIFT);
      ColorCode cc = c >> D_SHIFT;
      
      int treshold = D4[(x & D_PMASK) + ((y & D_PMASK) << D_PSHIFT)];  

      if((c & D_MASK) > treshold)
	cc++;

      *graph_ptr = *(Pixel24 *)&_truecolormap[cc];
    }
}

