
/*
 *
 * polyeng.C
 *
 */


#include "polyeng.h"
#include <string.h>

VertexPool PolygonEngine::_vpool;
Polygon    PolygonEngine::_p1;
Polygon    PolygonEngine::_p2;
Mug        PolygonEngine::_mug[PE_MAX_HEIGHT];

// Dithering matrix

int PolygonEngine::D4[16] =
   {
    0, 12,  3, 15,
    8,  4, 11,  7,
    2, 14,  1, 13,
   10,  6,  9,  5
   };


PolygonEngine::PolygonEngine(GraphicPort *gp, int verb) : GraphicProcessor(gp)
{
  _proc_name = "unknown PolygonEngine";
  Verbose(verb);
}

PolygonEngine::~PolygonEngine(void)
{
}

// Sutherland-Hodgman polygon _clipping

void PolygonEngine::ClipPoly(void)

{
  int i,j;
  int flag, old_flag;
  Vertex *S, *P, *I;
  int clip_z  = Attributes().Get(GA_ZBUFFER) || Attributes().Get(PEA_ZCLIP);

//  int k;
//  cerr << "clipping input\n";
//  for(k=0; k<_p1.Size(); k++)
//    cerr << k << "> x=" << _p1[k]->x << " y=" << _p1[k]->y << " z=" << _p1[k]->z << "\n";


  _vpool.Reset();

/* --------------- clip with x = _clip._x1 ---------------- */

  _p2.Reset();
  if(!_p1.Size())
    return;
  S = _p1[0];
  old_flag = S->x < _clip._x1;

  for(i=1; i<=_p1.Size(); i++)
    {
      j = i < _p1.Size() ? i : 0;
      P = _p1[j];
      flag = P->x < _clip._x1;

      if(flag ^ old_flag)
	{
	  I = _vpool.New();
	  I->flags.Set(VF_CLIP);
	  I->x = _clip._x1;
	  I->y = S->y + (long)(_clip._x1 - S->x) * (P->y - S->y) / (P->x - S->x);

	  if(Attributes().Get(GA_GOURAUD))
	    {
	      if(Attributes().Get(GA_RGB))
		{
		  I->r = S->r + (long)(_clip._x1 - S->x) * (P->r - S->r) / (P->x - S->x);
		  I->g = S->g + (long)(_clip._x1 - S->x) * (P->g - S->g) / (P->x - S->x);
		  I->b = S->b + (long)(_clip._x1 - S->x) * (P->b - S->b) / (P->x - S->x);
		}
	      else
		I->c = S->c + (long)(_clip._x1 - S->x) * (P->c - S->c) / (P->x - S->x);
	    }

	  if(Attributes().Get(GA_TEXTURE))
	    {
	      I->X = S->X + (long)(_clip._x1 - S->x) * (P->X - S->X) / (P->x - S->x);
	      I->Y = S->Y + (long)(_clip._x1 - S->x) * (P->Y - S->Y) / (P->x - S->x);
	    }

	  if(clip_z)
	    {
	      I->z = S->z + (long)(_clip._x1 - S->x) * (P->z - S->z) / (P->x - S->x);
	    }

          _p2.Push(I);
	}

      if(!flag)
	_p2.Push(P);

      S = P;
      old_flag = flag;
    }


/* --------------- clip with x = _clip._x2 ---------------- */

  _p1.Reset();
  if(!_p2.Size())
    return;
  S = _p2[0];
  old_flag = S->x > _clip._x2;


  for(i=1; i<=_p2.Size(); i++)
    {
      j = i < _p2.Size() ? i : 0;
      P = _p2[j];
      flag = P->x > _clip._x2;

      if(flag ^ old_flag)
	{
	  I = _vpool.New();
	  I->flags.Set(VF_CLIP);

	  I->x = _clip._x2;
	  I->y = P->y - (long)(P->x - _clip._x2) * (P->y - S->y) / (P->x - S->x);

	  if(Attributes().Get(GA_GOURAUD))
	    {
	      if(Attributes().Get(GA_RGB))
		{
		  I->r = P->r - (long)(P->x - _clip._x2) * (P->r - S->r) / (P->x - S->x);
		  I->g = P->g - (long)(P->x - _clip._x2) * (P->g - S->g) / (P->x - S->x);
		  I->b = P->b - (long)(P->x - _clip._x2) * (P->b - S->b) / (P->x - S->x);
		}
	      else
		I->c = P->c - (long)(P->x - _clip._x2) * (P->c - S->c) / (P->x - S->x);
	    }

	  if(Attributes().Get(GA_TEXTURE))
	    {
	      I->X = P->X - (long)(P->x - _clip._x2) * (P->X - S->X) / (P->x - S->x);
	      I->Y = P->Y - (long)(P->x - _clip._x2) * (P->Y - S->Y) / (P->x - S->x);
	    }

	  if(clip_z)
	    {
	      I->z = P->z - (long)(P->x - _clip._x2) * (P->z - S->z) / (P->x - S->x);
	    }

	  _p1.Push(I);
	}

      if(!flag)
	_p1.Push(P);

      S = P;
      old_flag = flag;
    }

/* ---------------  clip with y = _clip._y1 ---------------- */

  _p2.Reset();
  if(!_p1.Size())
    return;
  S = _p1[0];
  old_flag = S->y < _clip._y1;


  for(i=1; i<=_p1.Size(); i++)
    {

      j = i < _p1.Size() ? i : 0;


      P = _p1[j];
      flag = P->y < _clip._y1;

      if(flag ^ old_flag)
	{
	
	  I = _vpool.New();
	  I->flags.Set(VF_CLIP);

	  I->y = _clip._y1;
	  I->x = S->x + (long)(_clip._y1 - S->y) * (P->x - S->x) / (P->y - S->y);

	  if(Attributes().Get(GA_GOURAUD))
	    {
	      if(Attributes().Get(GA_RGB))
		{
		  I->r = S->r + (long)(_clip._y1 - S->y) * (P->r - S->r) / (P->y - S->y);
		  I->g = S->g + (long)(_clip._y1 - S->y) * (P->g - S->g) / (P->y - S->y);
		  I->b = S->b + (long)(_clip._y1 - S->y) * (P->b - S->b) / (P->y - S->y);
		}
	      else
		I->c = S->c + (long)(_clip._y1 - S->y) * (P->c - S->c) / (P->y - S->y);
	    }

	  if(Attributes().Get(GA_TEXTURE))
	    {
	      I->X = S->X + (long)(_clip._y1 - S->y) * (P->X - S->X) / (P->y - S->y);
	      I->Y = S->Y + (long)(_clip._y1 - S->y) * (P->Y - S->Y) / (P->y - S->y);
	    }

	  if(clip_z)
	    {
	      I->z = S->z + (long)(_clip._y1 - S->y) * (P->z - S->z) / (P->y - S->y);
	    }

          _p2.Push(I);
	}

      if(!flag)
	_p2.Push(P);

      S = P;
      old_flag = flag;
    }

/* --------------- clip with y = _clip._y2 ---------------- */

  _p1.Reset();
  if(!_p2.Size())
    return;
  S = _p2[0];
  old_flag = S->y > _clip._y2;


  for(i=1; i<=_p2.Size(); i++)
    {
      j = i < _p2.Size() ? i : 0;
      P = _p2[j];
      flag = P->y > _clip._y2;

      if(flag ^ old_flag)
	{
	  I = _vpool.New();
	  I->flags.Set(VF_CLIP);

	  I->y = _clip._y2;
	  I->x = P->x - (long)(P->y - _clip._y2) * (P->x - S->x) / (P->y - S->y);

	  if(Attributes().Get(GA_GOURAUD))
	    {
	      if(Attributes().Get(GA_RGB))
		{
		  I->r = P->r - (long)(P->y - _clip._y2) * (P->r - S->r) / (P->y - S->y);
		  I->g = P->g - (long)(P->y - _clip._y2) * (P->g - S->g) / (P->y - S->y);
		  I->b = P->b - (long)(P->y - _clip._y2) * (P->b - S->b) / (P->y - S->y);
		}
	      else
		I->c = P->c - (long)(P->y - _clip._y2) * (P->c - S->c) / (P->y - S->y);
	    }

	  if(Attributes().Get(GA_TEXTURE))
	    {
	      I->X = P->X - (long)(P->y - _clip._y2) * (P->X - S->X) / (P->y - S->y);
	      I->Y = P->Y - (long)(P->y - _clip._y2) * (P->Y - S->Y) / (P->y - S->y);
	    }

	  if(clip_z)
	    {
	      I->z = P->z - (long)(P->y - _clip._y2) * (P->z - S->z) / (P->y - S->y);
	    }

	  _p1.Push(I);
	}

      if(!flag)
	_p1.Push(P);

      S = P;
      old_flag = flag;
    }

  if(!(Attributes().Get(PEA_ZCLIP)))
    return;

/* ============= 3D clipping  ============== */

/* --------------- clip with z = _clip._z1 ---------------- */

  _p2.Reset();
  if(!_p1.Size())
    return;
  S = _p1[0];
  old_flag = S->z < _clip._z1;


  for(i=1; i<=_p1.Size(); i++)
    {

      j = i < _p1.Size() ? i : 0;


      P = _p1[j];
      flag = P->z < _clip._z1;

      if(flag ^ old_flag)
	{
	
	  I = _vpool.New();
	  I->flags.Set(VF_CLIP);

	  I->z = _clip._z1;
	  I->x = S->x + (long)(_clip._z1 - S->z) * (P->x - S->x) / (P->z - S->z);
	  I->y = S->y + (long)(_clip._z1 - S->z) * (P->y - S->y) / (P->z - S->z);

	  if(Attributes().Get(GA_GOURAUD))
	    {
	      if(Attributes().Get(GA_RGB))
		{
		  I->r = S->r + (long)(_clip._z1 - S->z) * (P->r - S->r) / (P->z - S->z);
		  I->g = S->g + (long)(_clip._z1 - S->z) * (P->g - S->g) / (P->z - S->z);
		  I->b = S->b + (long)(_clip._z1 - S->z) * (P->b - S->b) / (P->z - S->z);
		}
	      else
		I->c = S->c + (long)(_clip._z1 - S->z) * (P->c - S->c) / (P->z - S->z);
	    }

	  if(Attributes().Get(GA_TEXTURE))
	    {
	      I->X = S->X + (long)(_clip._z1 - S->z) * (P->X - S->X) / (P->z - S->z);
	      I->Y = S->Y + (long)(_clip._z1 - S->z) * (P->Y - S->Y) / (P->z - S->z);
	    }

          _p2.Push(I);
	}

      if(!flag)
	_p2.Push(P);

      S = P;
      old_flag = flag;
    }


/* --------------- clip with z = _clip._z2 ---------------- */

  _p1.Reset();
  if(!_p2.Size())
    return;
  S = _p2[0];
  old_flag = S->z > _clip._z2;


  for(i=1; i<=_p2.Size(); i++)
    {
      j = i < _p2.Size() ? i : 0;
      P = _p2[j];
      flag = P->z > _clip._z2;

      if(flag ^ old_flag)
	{
	  I = _vpool.New();
	  I->flags.Set(VF_CLIP);

	  I->z = _clip._z2;
	  I->x = P->x - (long)(P->z - _clip._z2) * (P->x - S->x) / (P->z - S->z);
	  I->y = P->y - (long)(P->z - _clip._z2) * (P->y - S->y) / (P->z - S->z);

	  if(Attributes().Get(GA_GOURAUD))
	    {
	      if(Attributes().Get(GA_RGB))
		{
		  I->r = P->r - (long)(P->z - _clip._z2) * (P->r - S->r) / (P->z - S->z);
		  I->g = P->g - (long)(P->z - _clip._z2) * (P->g - S->g) / (P->z - S->z);
		  I->b = P->b - (long)(P->z - _clip._z2) * (P->b - S->b) / (P->z - S->z);
		}
	      else
		I->c = P->c - (long)(P->z - _clip._z2) * (P->c - S->c) / (P->z - S->z);
	    }

	  if(Attributes().Get(GA_TEXTURE))
	    {
	      I->X = P->X - (long)(P->z - _clip._z2) * (P->X - S->X) / (P->z - S->z);
	      I->Y = P->Y - (long)(P->z - _clip._z2) * (P->Y - S->Y) / (P->z - S->z);
	    }

	  _p1.Push(I);
	}

      if(!flag)
	_p1.Push(P);

      S = P;
      old_flag = flag;
    }

//  cerr << "clipping output\n";
//  for(k=0; k<_p1.Size(); k++)
//    cerr << k << "> x=" << _p1[k]->x << " y=" << _p1[k]->y << " z=" << _p1[k]->z << "\n";

//  cerr << _vpool;
}


int PolygonEngine_VCC::_entries = 0;

PolygonEngine_VCC::PolygonEngine_VCC(void)
{
}


void PolygonEngine_VCC::Register(const PolygonEngine_MakeFun make)
{
  assert(_entries < VC_ENTRIES);
  _entry[_entries] = make;
  _entries++;

#ifdef EBUG
  cerr << "[TAGL_VC] PolygonEngine_VCC::register() called - " << _entries << " entries\n";
#endif
}

PolygonEngine* PolygonEngine_VCC::Make(GraphicPort *GP, int verb)
{
  int i;
  PolygonEngine *PE;

  for(i=0; i<_entries; i++)
      if(PE = _entry[i](GP,verb))
	return PE;
  
  return NULL;
}

PolygonEngine_VCC PolygonEngine_VC;

PolygonEngine* MakePolygonEngine(GraphicPort *GP, int verb)
{
  return PolygonEngine_VC.Make(GP, verb);
}
